# Copyright (C) 2012 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Jan Safranek <jsafrane@redhat.com>
# -*- coding: utf-8 -*-
"""
Module for LMI_GenericDiskPartition class.

LMI_GenericDiskPartition
------------------------

.. autoclass:: LMI_GenericDiskPartition
    :members:

"""

from lmi.storage.ExtentProvider import ExtentProvider
import blivet
import lmi.storage.util.storage as storage
import lmi.providers.cmpi_logging as cmpi_logging

LOG = cmpi_logging.get_logger(__name__)

class LMI_GenericDiskPartition(ExtentProvider):
    """
        Provider of LMI_GenericDiskPartition class.
    """

    @cmpi_logging.trace_method
    def __init__(self, *args, **kwargs):
        super(LMI_GenericDiskPartition, self).__init__(
                'LMI_GenericDiskPartition', *args, **kwargs)


    @cmpi_logging.trace_method

    def provides_device(self, device):
        """
            Returns True, if this class is provider for given Anaconda
            StorageDevice class.
        """
        if isinstance(device, blivet.devices.PartitionDevice):
            if device.parents[0].format.labelType == 'msdos':
                return False
            return True
        return False

    @cmpi_logging.trace_method
    def enumerate_devices(self):
        """
            Enumerate all StorageDevices, that this provider provides.
        """
        for device in self.storage.partitions:
            if self.provides_device(device):
                yield device

    @cmpi_logging.trace_method
    def do_delete_instance(self, device):
        """
            Really delete given Anaconda StorageDevice.
        """
        LOG().info("DELETE PARTITION: %s", device.path)
        storage.remove_partition(self.storage, device)



# Copyright (C) 2012 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Jan Safranek <jsafrane@redhat.com>
# -*- coding: utf-8 -*-
"""
Module for SettingHelper class.

SettingHelper
-------------

.. autoclass:: SettingHelper
    :members:

"""

import lmi.providers.cmpi_logging as cmpi_logging

class SettingHelper(object):
    """
        Abstract interface to help implementation of LMI_*Setting classes.
        LMI_*Setting instances are associated with some managed
        element, let's say LMI_Foo. LMI_Foo has its own provider, which can
        enumerate and provide LMI_Foo instances. If LMI_Foo inherits also
        SettingHelper, it can then enumerate and provider CIM_FooSetting
        instances associated to the managed foos.
    """

    @cmpi_logging.trace_method
    def __init__(self, setting_classname, *args, **kwargs):
        self.setting_classname = setting_classname
        super(SettingHelper, self).__init__(*args, **kwargs)

    @cmpi_logging.trace_method
    # pylint: disable-msg=W0613
    def enumerate_settings(self, setting_provider):
        """
            This method returns iterable with all instances of LMI_*Setting
            as Setting instances.
        """
        return []

    @cmpi_logging.trace_method
    # pylint: disable-msg=W0613
    def get_setting_for_id(self, setting_provider, instance_id):
        """
            Return Setting instance, which corresponds to LMI_*Setting with
            given InstanceID.
            Return None if there is no such instance.

            Subclasses must override this method.
        """
        return None

    @cmpi_logging.trace_method
    # pylint: disable-msg=W0613
    def get_associated_element_name(self, setting_provider, instance_id):
        """
            Return CIMInstanceName of ManagedElement for ElementSettingData
            association for setting with given ID.
            Return None if no such ManagedElement exists.
        """
        return None

    @cmpi_logging.trace_method
    # pylint: disable-msg=W0613
    def get_supported_setting_properties(self, setting_provider):
        """
            Return hash property_name -> constructor.
                constructor is a function which takes string argument
                and returns CIM value. (i.e. pywbem.Uint16
                or bool or string etc).
            This hash will be passed to SettingProvider.__init__
        """
        return {}

    @cmpi_logging.trace_method
    # pylint: disable-msg=W0613
    def get_setting_ignore(self, setting_provider):
        """
            Return hash property_name -> default value.
            If this value of the property is set, the ModifyInstance
            won't complain, but it will silently ignore the value.
            This is useful when someone tries to set default value
            of a property and the provider does not implement it.
        """
        return None

    @cmpi_logging.trace_method
    # pylint: disable-msg=W0613
    def get_setting_validators(self, setting_provider):
        """
            Return hash property_name -> validator
                validator is a function which takes pywbem (Uint32, bool ...)
                value as parameter and returns True, if the value is correct for
                the property. Not all properties do need to have a validator.
        """
        return None



#!/usr/bin/python
#
# Copyright (C) 2013 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Jan Synacek <jsynacek@redhat.com>
#          Jan Safranek <jsafrane@redhat.com>

from test_base import StorageTestBase, short_tests_only
import unittest
import pywbem
import subprocess
import sys
import traceback

class TestLUKS(StorageTestBase):
    @classmethod
    def setUpClass(cls):
        super(TestLUKS, cls).setUpClass()
        cls.luks_partition = cls.partition_names[-1]
        cls.luks_passphrase = '23rnv0as0df23n'
        cls.luks_newpassphrase = 'v939tnmnopxc09'
        cls.luks_elementname = 'luks-cim-test'

    def setUp(self):
        super(TestLUKS, self).setUp()
        self.service = self.wbemconnection.EnumerateInstanceNames("LMI_ExtentEncryptionConfigurationService")[0]

    def _create_luks(self, device, passphrase):
        """
        Create LUKS format on given device.
        
        :param device: CIMInstanceName
        :param passphrase: string
        :returns: LMI_EncryptionFormat
        """
        (ret, outparams) = self.invoke_async_method("CreateEncryptionFormat",
                self.service,
               int,
                InExtent=device,
                Passphrase=passphrase)
        self.assertEquals(ret, 0)
        return outparams['Format']

    def test_create_encryption_format(self):
        """
        Test to create, open, close and delete one LUKS.
        """
        fmt = self._create_luks(self.luks_partition, self.luks_passphrase)
        self.assertTrue(fmt)
        opened = False

        try:
            # check the returned format
            self.assertEquals(fmt['CSName'], self.SYSTEM_NAME)
            self.assertEquals(fmt['CSCreationClassName'], self.SYSTEM_CLASS_NAME)
            self.assertTrue(fmt['Name'].startswith("UUID="))
            self.assertEquals(fmt['CreationClassName'], "LMI_LUKSFormat")

            # check it's associated to the original device
            devices = self.wbemconnection.AssociatorNames(fmt, AssocClass="LMI_ResidesOnExtent")
            self.assertEquals(len(devices), 1)
            device = devices[0]
            self.assertCIMNameEquals(device, self.luks_partition)

            # open it using our password
            (ret, outparams) = self.invoke_async_method(
                    "OpenEncryptionFormat",
                    self.service,
                    int,
                    ElementName=self.luks_elementname,
                    Passphrase=self.luks_passphrase,
                    Format=fmt)
            self.assertEquals(ret, 0)
            opened = True
            extent_name = outparams['Extent']
            self.assertTrue(extent_name)
            extent = self.wbemconnection.GetInstance(extent_name)

            self.assertEquals(extent['ElementName'], self.luks_elementname)
            self.assertEquals(extent['SystemName'], self.SYSTEM_NAME)
            self.assertEquals(extent['SystemCreationClassName'], self.SYSTEM_CLASS_NAME)
            self.assertEquals(extent['CreationClassName'], "LMI_LUKSStorageExtent")

            # check the extent is associated properly
            basedons = self.wbemconnection.AssociatorNames(extent_name, AssocClass="LMI_LUKSBasedOn")
            self.assertEquals(len(basedons), 1)
            basedon = basedons[0]
            self.assertCIMNameEquals(basedon, device)

        except:
            # print the exception so it's not overwritten by any new exception
            # in 'finally' block
            print >> sys.stderr, "test_create_encryption_format failed:"
            print >> sys.stderr, traceback.format_exc()
            raise

        finally:
            # close the device
            if opened:
                (ret, outparams) = self.invoke_async_method(
                        "CloseEncryptionFormat",
                        self.service,
                        int,
                        Format=fmt)
                self.assertEquals(ret, 0)


    def test_add_and_delete_passphrase(self):
        """
        Test adding and removing passphrases.
        """
        USED = 1  # LMI_LUKSFormat.SlotStatus.used
        UNUSED = 0
        passphrases = [self.luks_newpassphrase + str(x) for x in range(7)]

        fmt = self._create_luks(self.luks_partition, self.luks_passphrase)
        self.assertTrue(fmt)

        # add 7 different passwords
        for i in xrange(7):
            f = self.wbemconnection.GetInstance(fmt)
            self.assertListEqual(f['SlotStatus'], [USED] * (i + 1) + [UNUSED] * (7 - i))
            (ret, outparams) = self.wbemconnection.InvokeMethod(
                "AddPassphrase",
                self.service,
                NewPassphrase=passphrases[i],
                Passphrase=self.luks_passphrase,
                Format=fmt)
            self.assertEquals(ret, 0)
        f = self.wbemconnection.GetInstance(fmt)
        self.assertListEqual(f['SlotStatus'], [USED] * 8)

        # add 9th password -> error
        self.assertRaises(pywbem.CIMError, self.wbemconnection.InvokeMethod,
                "AddPassphrase",
                self.service,
                NewPassphrase=self.luks_newpassphrase * 2,
                Passphrase=self.luks_passphrase,
                Format=fmt)

        # remove the added passwords
        for i in xrange(7):
            f = self.wbemconnection.GetInstance(fmt)
            self.assertListEqual(f['SlotStatus'], [USED] + [UNUSED] * i + [USED] * (7 - i))
            (ret, outparams) = self.wbemconnection.InvokeMethod(
                "DeletePassphrase",
                self.service,
                Passphrase=passphrases[i],
                Format=fmt)
            self.assertEquals(ret, 0)

        f = self.wbemconnection.GetInstance(fmt)
        self.assertListEqual(f['SlotStatus'], [USED] + [UNUSED] * 7)

        # remove removed password -> error
        self.assertRaises(pywbem.CIMError, self.wbemconnection.InvokeMethod,
                "DeletePassphrase",
                self.service,
                Passphrase=passphrases[0],
                Format=fmt)


    def test_delete_wrong_passphrase(self):
        """
        Test removal of wrong passphrase.
        """
        fmt = self._create_luks(self.luks_partition, self.luks_passphrase)
        self.assertTrue(fmt)

        self.assertRaises(
            pywbem.CIMError,
            self.wbemconnection.InvokeMethod,
            "DeletePassphrase",
            self.service,
            Passphrase='NoSuchPassword',
            Format=fmt)

    def test_double_open_encryption_format(self):
        """
        Test opening already opened LUKS.
        """
        fmt = self._create_luks(self.luks_partition, self.luks_passphrase)
        self.assertTrue(fmt)

        (ret, outparams) = self.invoke_async_method(
                "OpenEncryptionFormat",
                self.service,
                int,
                ElementName=self.luks_elementname,
                Passphrase=self.luks_passphrase,
                Format=fmt)
        self.assertEquals(ret, 0)

        try:
            self.assertRaises(
                    pywbem.CIMError,
                    self.invoke_async_method,
                    "OpenEncryptionFormat",
                    self.service,
                    int,
                    ElementName=self.luks_elementname,
                    Passphrase=self.luks_passphrase,
                    Format=fmt)
        except:
            # print the exception so it's not overwritten by any new exception
            # in 'finally' block
            print >> sys.stderr, "test_create_encryption_format failed:"
            print >> sys.stderr, traceback.format_exc()
            raise

        finally:
            (ret, outparams) = self.invoke_async_method(
                    "CloseEncryptionFormat",
                    self.service,
                    int,
                    Format=fmt)
            self.assertEquals(ret, 0)


    def test_double_close_encryption_format(self):
        """
        Test closing already closed LUKS.
        """
        fmt = self._create_luks(self.luks_partition, self.luks_passphrase)
        self.assertTrue(fmt)

        (ret, outparams) = self.invoke_async_method(
                "OpenEncryptionFormat",
                self.service,
                int,
                ElementName=self.luks_elementname,
                Passphrase=self.luks_passphrase,
                Format=fmt)
        self.assertEquals(ret, 0)

        (ret, outparams) = self.invoke_async_method(
                "CloseEncryptionFormat",
                self.service,
                int,
                Format=fmt)
        self.assertEquals(ret, 0)

        self.assertRaises(
            pywbem.CIMError,
            self.invoke_async_method,
            "CloseEncryptionFormat",
            self.service,
            int,
            Format=fmt)


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2012 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Jan Safranek <jsafrane@redhat.com>
# -*- coding: utf-8 -*-
"""
Module for LMI_PartitionBasedOn class.

LMI_PartitionBasedOn
--------------------

.. autoclass:: LMI_PartitionBasedOn
    :members:

"""

from lmi.storage.BasedOnProvider import BasedOnProvider
import pywbem
import lmi.storage.util.storage as storage
import lmi.providers.cmpi_logging as cmpi_logging

class LMI_PartitionBasedOn(BasedOnProvider):
    """
        Implementation of BasedOn class.
    """
    @cmpi_logging.trace_method
    def __init__(self, *args, **kwargs):
        super(LMI_PartitionBasedOn, self).__init__(*args, **kwargs)


    @cmpi_logging.trace_method

    def enumerate_devices(self):
        """
            Enumerate all devices, which are in this association as
            Dependent ones, i.e. all devices, which do not have any
            specialized BasedOn class
        """
        return self.storage.partitions

    @cmpi_logging.trace_method
    def get_logical_partition_start(self, device):
        """
            Return starting address of logical's partition metadata.
        """
        return storage.get_logical_partition_start(device)

    @cmpi_logging.trace_method
    def get_mbr_instance(self, model, device, base):
        """ Fill instance of PartitionBasedOn class with MBR positions. """
        # primary partitions are simple
        if device.isPrimary or device.isExtended:
            return self.get_gpt_instance(model, device, base)

        # startaddress is relative to the beginning of the extended partition
        base_start = base.partedPartition.geometry.start
        # find the metadata
        start = self.get_logical_partition_start(device)
        end = device.partedPartition.geometry.end

        model['OrderIndex'] = pywbem.Uint16(device.partedPartition.number)
        model['StartingAddress'] = pywbem.Uint64(start - base_start)
        model['EndingAddress'] = pywbem.Uint64(end - base_start)
        return model

    @cmpi_logging.trace_method
    # pylint: disable-msg=W0613
    def get_gpt_instance(self, model, device, base):
        """ Fill instance of PartitionBasedOn class with GPT positions. """
        model['OrderIndex'] = pywbem.Uint16(device.partedPartition.number)
        model['StartingAddress'] = \
            pywbem.Uint64(device.partedPartition.geometry.start)
        model['EndingAddress'] = \
            pywbem.Uint64(device.partedPartition.geometry.end)
        return model

    @cmpi_logging.trace_method
    def get_instance(self, env, model, device=None, base=None):
        model = super(LMI_PartitionBasedOn, self).get_instance(
                env, model, device, base)

        if not device:
            device = self.provider_manager.get_device_for_name(
                    model['Dependent'])
        if not base:
            base = self.provider_manager.get_device_for_name(
                    model['Antecedent'])

        if device.isLogical:
            model = self.get_mbr_instance(model, device, base)
        elif base.format.labelType == 'msdos':
            model = self.get_mbr_instance(model, device, base)
        elif base.format.labelType == 'gpt':
            model = self.get_gpt_instance(model, device, base)
        return model



# Copyright (C) 2012 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Jan Safranek <jsafrane@redhat.com>
# -*- coding: utf-8 -*-
"""
Module for DeviceProvider class.

DeviceProvider
--------------

.. autoclass:: DeviceProvider
    :members:

"""

from lmi.storage.BaseProvider import BaseProvider
import pywbem
import lmi.providers.cmpi_logging as cmpi_logging

LOG = cmpi_logging.get_logger(__name__)

class DeviceProvider(BaseProvider):
    """
        CIM Provider which provides CIM StorageExtent or CIM_StoragePool
        of a Anaconda device.

        In addition to CIM provider methods, this class and its subclasses
        can convert CIM InstanceName to Anaconda's StorageDevice instance
        and a vice versa.
    """
    @cmpi_logging.trace_method
    def __init__(self, *args, **kwargs):
        """
            Initialize the provider.
            Store reference to blivet.Blivet.
            Store reference to StorageConfiguration.
            Register at given ProviderManager.
        """
        super(DeviceProvider, self).__init__(*args, **kwargs)
        self.provider_manager.add_device_provider(self)

    @cmpi_logging.trace_method
    # pylint: disable-msg=W0613
    def provides_name(self, object_name):
        """
            Returns True, if this class is provider for given CIM InstanceName.
        """
        return False

    @cmpi_logging.trace_method
    # pylint: disable-msg=W0613
    def provides_device(self, device):
        """
            Returns True, if this class is provider for given Anaconda
            StorageDevice class.
        """
        return False

    @cmpi_logging.trace_method
    # pylint: disable-msg=W0613
    def get_device_for_name(self, object_name):
        """
            Returns Anaconda StorageDevice for given CIM InstanceName or
            None if no device is found.
        """
        return None

    @cmpi_logging.trace_method
    # pylint: disable-msg=W0613
    def get_name_for_device(self, device):
        """
            Returns CIM InstanceName for given Anaconda StorageDevice.
            None if no device is found.
        """
        return None

    @cmpi_logging.trace_method
    def get_status(self, device):
        """
            Returns OperationalStatus for given Anaconda StorageDevice.
            It combines statuses of all parent devices.
            Subclasses should override this method to provide additional
            statuses.
        """
        status = set()
        parents = self.get_base_devices(device)
        if len(parents) > 0:
            for parent in parents:
                parent_provider = self.provider_manager.get_provider_for_device(
                        parent)
                parent_status = parent_provider.get_status(parent)
                status.update(parent_status)
        else:
            status.add(self.Values.OperationalStatus.OK)
        return list(status)

    @cmpi_logging.trace_method
    def get_base_devices(self, device):
        """
            Return iterable with base devices for given StorageDevice.
            Base devices are StorageDevices, that the given StorageDevice
            depend on, e.g. RAID members of a RAID, physical volumes
            of a Volume Group and Volume Group of Logical Volume.
        """
        if device.parents:
            return device.parents
        return []

    @cmpi_logging.trace_method
    def _find_redundancy(self, device):
        """
            Discover redundancy of given StorageDevice.
            It uses ProviderManager to do so.
        """
        provider = self.provider_manager.get_provider_for_device(device)
        if not provider:
            raise pywbem.CIMError(pywbem.CIM_ERR_FAILED,
                    "Cannot find provider for device " + device.path)
        return provider.get_redundancy(device)

    @cmpi_logging.trace_method
    # pylint: disable-msg=W0613
    def do_delete_instance(self, device):
        """
            Really delete given Anaconda StorageDevice.

            Subclasses must override this method to allow DeleteInstance
            intrinsic method.
        """
        raise pywbem.CIMError(pywbem.CIM_ERR_NOT_SUPPORTED,
                    "Cannot delete this device.")

    @cmpi_logging.trace_method
    def delete_instance(self, env, instance_name):
        """Delete an instance intrinsic method"""
        # just check the instance validity
        device = self.get_device_for_name(instance_name)
        if not device:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND,
                    "Cannot find the device.")
        if self.storage.deviceDeps(device):
            deps = self.storage.deviceDeps(device)
            depnames = [device.path for device in deps]
            LOG().info("Cannot remove %s, it's used by %s"
                    % (device.path, str(depnames)))
            raise pywbem.CIMError(pywbem.CIM_ERR_FAILED,
                    "The device is in use by %s" % str(depnames))

        self.do_delete_instance(device)

    @cmpi_logging.trace_method
    def get_redundancy(self, device):
        """
            Returns redundancy characteristics for given Anaconda StorageDevice.
        """
        parents = self.get_base_devices(device)
        if len(parents) > 0:
            # find all parents and get their redundancy
            redundancies = [self._find_redundancy(device) for device in parents]
            # iteratively call self.get_common_redundancy(r1, r2), ...
            final_redundancy = self.Redundancy.get_common_redundancy_list(
                    redundancies)
        else:
            # this device has no parents, assume it is simple disk
            final_redundancy = self.Redundancy(
                    no_single_point_of_failure=False,
                    data_redundancy=1,
                    package_redundancy=0,
                    stripe_length=1)
        return final_redundancy

    class Redundancy(object):
        """
            Class representing redundancy characteristics of a StorageExtent
            device, i.e. both StorageExtent and StoragePool
        """

        # constants for RAID levels
        RAID0 = 0
        RAID1 = 1
        RAID4 = 4
        RAID5 = 5
        RAID6 = 6
        RAID10 = 10
        LINEAR = 4096

        # ParityLayout
        PARITY_ROTATED = 2
        PARITY_NON_ROTATED = 1

        @cmpi_logging.trace_method
        def __init__(self, no_single_point_of_failure=False,
                     data_redundancy=1,
                     package_redundancy=0,
                     stripe_length=1,
                     parity_layout=None):
            self.no_single_point_of_failure = no_single_point_of_failure
            self.data_redundancy = data_redundancy
            self.package_redundancy = package_redundancy
            self.stripe_length = stripe_length
            self.parity_layout = parity_layout

        @cmpi_logging.trace_method
        def get_redundancy_raid0(self, second):
            """
                Return the combined data redundancy characteristics for
                two devices combined in RAID0.
            """
            # data is spread on all devices -> DataRedundancy is sum of base
            # DataRedundancies
            # PackageRedundancy is the minimum of PackageRedundancies
            data_redundancy = min(self.data_redundancy, second.data_redundancy)
            package_redundancy = min(self.package_redundancy,
                    second.package_redundancy)
            no_single_point_of_failure = (self.no_single_point_of_failure
                    and second.no_single_point_of_failure)
            stripe_length = self.stripe_length + second.stripe_length

            return DeviceProvider.Redundancy(
                    no_single_point_of_failure=no_single_point_of_failure,
                    data_redundancy=data_redundancy,
                    package_redundancy=package_redundancy,
                    stripe_length=stripe_length)

        @cmpi_logging.trace_method
        def get_redundancy_raid1(self, second):
            """
                Return the combined data redundancy characteristics for
                two devices combined in RAID1.
            """
            data_redundancy = self.data_redundancy + second.data_redundancy
            package_redundancy = (self.package_redundancy
                    + second.package_redundancy)
            no_single_point_of_failure = True
            stripe_length = min(self.stripe_length, second.stripe_length)

            return DeviceProvider.Redundancy(
                    no_single_point_of_failure=no_single_point_of_failure,
                    data_redundancy=data_redundancy,
                    package_redundancy=package_redundancy,
                    stripe_length=stripe_length)

        @cmpi_logging.trace_method
        def get_redundancy_raid5(self, second):
            """
                Return the combined data redundancy characteristics for
                two devices combined in RAID5.
            """
            data_redundancy = min(self.data_redundancy, second.data_redundancy)
            package_redundancy = min(self.package_redundancy,
                    second.package_redundancy)
            no_single_point_of_failure = True
            stripe_length = self.stripe_length + second.stripe_length

            return DeviceProvider.Redundancy(
                    no_single_point_of_failure=no_single_point_of_failure,
                    data_redundancy=data_redundancy,
                    package_redundancy=package_redundancy,
                    stripe_length=stripe_length,
                    parity_layout=self.PARITY_ROTATED)

        @cmpi_logging.trace_method
        def get_redundancy_raid4(self, second):
            """
                Return the combined data redundancy characteristics for
                two devices combined in RAID4.
            """
            redundancy = self.get_redundancy_raid5(second)
            redundancy.parity_layout = self.PARITY_NON_ROTATED
            return redundancy


        @cmpi_logging.trace_method
        def get_redundancy_raid6(self, second):
            """
                Return the combined data redundancy characteristics for
                two devices combined in RAID6.
            """
            data_redundancy = min(self.data_redundancy, second.data_redundancy)
            package_redundancy = min(self.package_redundancy,
                    second.package_redundancy)
            no_single_point_of_failure = True
            stripe_length = self.stripe_length + second.stripe_length

            return DeviceProvider.Redundancy(
                    no_single_point_of_failure=no_single_point_of_failure,
                    data_redundancy=data_redundancy,
                    package_redundancy=package_redundancy,
                    stripe_length=stripe_length,
                    parity_layout=self.PARITY_ROTATED)

        @cmpi_logging.trace_method
        def get_redundancy_raid10(self, second):
            """
                Return the combined data redundancy characteristics for
                two devices combined in RAID10.
            """
            data_redundancy = self.data_redundancy + second.data_redundancy
            package_redundancy = min(self.package_redundancy,
                    second.package_redundancy)
            no_single_point_of_failure = True
            stripe_length = self.stripe_length + second.stripe_length

            return DeviceProvider.Redundancy(
                    no_single_point_of_failure=no_single_point_of_failure,
                    data_redundancy=data_redundancy,
                    package_redundancy=package_redundancy,
                    stripe_length=stripe_length)


        @cmpi_logging.trace_method
        def get_redundancy_linear(self, second):
            """
                Return the combined data redundancy characteristics for
                two devices.
                Linear device is assumed, i.e. the data are either on self or
                on B.
            """
            # assume linear device, i.e. a data is either on A or on B
            # hence data_redundancy is the minimum of both
            data_redundancy = min(self.data_redundancy, second.data_redundancy)
            # assume the worst
            package_redundancy = min(self.package_redundancy,
                    second.package_redundancy)
            # both NoSinglePointOfFailure must be true to be the result true
            no_single_point_of_failure = (self.no_single_point_of_failure
                    and second.no_single_point_of_failure)
            #  we don't know if the data are on A or B, so assume the worst
            stripe_length = min(self.stripe_length, second.stripe_length)
            return DeviceProvider.Redundancy(
                    no_single_point_of_failure=no_single_point_of_failure,
                    data_redundancy=data_redundancy,
                    package_redundancy=package_redundancy,
                    stripe_length=stripe_length)

        @staticmethod
        @cmpi_logging.trace_function
        def get_common_redundancy_list(redundancy_list,
                 raid_level=LINEAR):
            """
                Return common redundancy characteristics for list of devices.
                Linear device is assumed, i.e. the data are either on self or
                on B.

                raid_level: LINEAR = Linear, 0,1,5,6 - raidX
            """
            if raid_level == DeviceProvider.Redundancy.LINEAR:
                redundancy = reduce(
                        lambda a, b: a.get_redundancy_linear(b),
                        redundancy_list)
            elif raid_level == DeviceProvider.Redundancy.RAID0:
                redundancy = reduce(
                        lambda a, b: a.get_redundancy_raid0(b),
                        redundancy_list)
            elif raid_level == DeviceProvider.Redundancy.RAID1:
                redundancy = reduce(
                        lambda a, b: a.get_redundancy_raid1(b),
                        redundancy_list)
                redundancy.package_redundancy = \
                        redundancy.package_redundancy + len(redundancy_list) - 1
            elif raid_level == DeviceProvider.Redundancy.RAID4:
                redundancy = reduce(
                        lambda a, b: a.get_redundancy_raid4(b),
                        redundancy_list)
                redundancy.package_redundancy = \
                    redundancy.package_redundancy + 1
            elif raid_level == DeviceProvider.Redundancy.RAID5:
                redundancy = reduce(
                        lambda a, b: a.get_redundancy_raid5(b),
                        redundancy_list)
                redundancy.package_redundancy = \
                    redundancy.package_redundancy + 1
            elif raid_level == DeviceProvider.Redundancy.RAID6:
                redundancy = reduce(
                        lambda a, b: a.get_redundancy_raid6(b),
                        redundancy_list)
                redundancy.package_redundancy = \
                    redundancy.package_redundancy + 2
            elif raid_level == DeviceProvider.Redundancy.RAID10:
                redundancy = reduce(
                        lambda a, b: a.get_redundancy_raid10(b),
                        redundancy_list)

                # stripe-length needs to be calculated separately
                # nr. of stripes for N=2: 1, N=3: 2, N=4: 2, N=5: 3, ...
                stripes = len(redundancy_list) // 2 + len(redundancy_list) % 2
                stripe_lengths = [x.stripe_length for x in redundancy_list]
                stripe_lengths.sort()
                # final stripe_length = sum of the 'stripes' lowest stripe
                # lengths
                stripe_lengths = stripe_lengths[:stripes]
                redundancy.stripe_length = reduce(
                        lambda a, b: a + b,
                        stripe_lengths)

                # data redundancy needs to be calculated separately
                # it's always 2 (at least for now),
                # i.e. sum of two lowest data redundancies
                data_redundancies = [x.data_redundancy for x in redundancy_list]
                data_redundancies.sort()
                redundancy.data_redundancy = (data_redundancies[0]
                        + data_redundancies[1])

                redundancy.package_redundancy = \
                    redundancy.package_redundancy + 1
            else:
                LOG().trace_warn("Unknown raid_level: " + str(raid_level))
                return None
            return redundancy

    class Values(object):
        class OperationalStatus(object):
            Unknown = pywbem.Uint16(0)
            Other = pywbem.Uint16(1)
            OK = pywbem.Uint16(2)
            Degraded = pywbem.Uint16(3)
            Stressed = pywbem.Uint16(4)
            Predictive_Failure = pywbem.Uint16(5)
            Error = pywbem.Uint16(6)
            Non_Recoverable_Error = pywbem.Uint16(7)
            Starting = pywbem.Uint16(8)
            Stopping = pywbem.Uint16(9)
            Stopped = pywbem.Uint16(10)
            In_Service = pywbem.Uint16(11)
            No_Contact = pywbem.Uint16(12)
            Lost_Communication = pywbem.Uint16(13)
            Aborted = pywbem.Uint16(14)
            Dormant = pywbem.Uint16(15)
            Supporting_Entity_in_Error = pywbem.Uint16(16)
            Completed = pywbem.Uint16(17)
            Power_Mode = pywbem.Uint16(18)
            Relocating = pywbem.Uint16(19)
            # DMTF_Reserved = ..
            # Vendor_Reserved = 0x8000..



# Copyright (C) 2012 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Jan Safranek <jsafrane@redhat.com>
# -*- coding: utf-8 -*-
"""
Module for LMI_LVStorageExtent class.

LMI_LVStorageExtent
-------------------

.. autoclass:: LMI_LVStorageExtent
    :members:

"""

from lmi.storage.ExtentProvider import ExtentProvider
import blivet
import lmi.providers.cmpi_logging as cmpi_logging
from lmi.storage.SettingHelper import SettingHelper
from lmi.storage.SettingManager import StorageSetting
import pywbem
import lmi.storage.util.storage as storage
from lmi.storage.SettingProvider import SettingProvider

LOG = cmpi_logging.get_logger(__name__)

class LMI_LVStorageExtent(ExtentProvider, SettingHelper):
    """
        Provider of LMI_LVStorageExtent class.
    """

    @cmpi_logging.trace_method
    def __init__(self, *args, **kwargs):
        super(LMI_LVStorageExtent, self).__init__(
                classname='LMI_LVStorageExtent',
                setting_classname='LMI_LVStorageSetting',
                *args, **kwargs)

    @cmpi_logging.trace_method
    def provides_device(self, device):
        """
            Returns True, if this class is provider for given Anaconda
            StorageDevice class.
        """
        if  isinstance(device,
                    blivet.devices.LVMLogicalVolumeDevice):
            return True
        return False

    @cmpi_logging.trace_method
    def enumerate_devices(self):
        """
            Enumerate all StorageDevices, that this provider provides.
        """
        for device in self.storage.lvs:
            yield device

    @cmpi_logging.trace_method
    def get_element_name(self, device):
        return device.lvname

    @cmpi_logging.trace_method
    def get_instance(self, env, model, device=None):
        """
            Add LV-specific properties.
        """
        model = super(LMI_LVStorageExtent, self).get_instance(
                env, model, device)
        if not device:
            device = self._get_device(model)

        model['UUID'] = device.uuid

        return model

    @cmpi_logging.trace_method
    def _get_setting_for_device(self, device, setting_provider):
        """ Return setting for given device """
        _id = storage.get_persistent_name(device)
        setting = self.setting_manager.create_setting(
                self.setting_classname,
                StorageSetting.TYPE_CONFIGURATION,
                setting_provider.create_setting_id(_id),
                class_to_create=StorageSetting)
        setting.set_setting(self.get_redundancy(device))
        setting['ElementName'] = device.path
        return setting

    @cmpi_logging.trace_method
    def enumerate_settings(self, setting_provider):
        """
            This method returns iterable with all instances of LMI_*Setting
            as Setting instances.
        """
        for lv in self.storage.lvs:
            yield self._get_setting_for_device(lv, setting_provider)

    @cmpi_logging.trace_method
    def get_setting_for_id(self, setting_provider, instance_id):
        """
            Return Setting instance, which corresponds to LMI_*Setting with
            given InstanceID.
            Return None if there is no such instance.

            Subclasses must override this method.
        """
        path = setting_provider.parse_setting_id(instance_id)
        if not path:
            return None
        device = storage.get_device_for_persistent_name(self.storage, path)
        if not path:
            return None
        if not isinstance(device,
                blivet.devices.LVMLogicalVolumeDevice):
            LOG().trace_warn("InstanceID %s is not LVMLogicalVolumeDevice",
                    instance_id)
            return None
        return self._get_setting_for_device(device, setting_provider)

    @cmpi_logging.trace_method
    def get_associated_element_name(self, setting_provider, instance_id):
        """
            Return CIMInstanceName of ManagedElement for ElementSettingData
            association for setting with given ID.
            Return None if no such ManagedElement exists.
        """
        path = setting_provider.parse_setting_id(instance_id)
        if not path:
            return None
        device = storage.get_device_for_persistent_name(self.storage, path)
        if not device:
            return None
        if not isinstance(device,
                blivet.devices.LVMLogicalVolumeDevice):
            LOG().trace_warn("InstanceID %s is not LVMLogicalVolumeDevice",
                    instance_id)
            return None
        return self.get_name_for_device(device)

    @cmpi_logging.trace_method
    def get_supported_setting_properties(self, setting_provider):
        """
            Return hash property_name -> constructor.
                constructor is a function which takes string argument
                and returns CIM value. (i.e. pywbem.Uint16
                or bool or string etc).
            This hash will be passed to SettingProvider.__init__
        """
        return {
                'DataRedundancyGoal': pywbem.Uint16,
                'DataRedundancyMax': pywbem.Uint16,
                'DataRedundancyMin': pywbem.Uint16,
                'ExtentStripeLength' : pywbem.Uint16,
                'ExtentStripeLengthMax' : pywbem.Uint16,
                'ExtentStripeLengthMin' : pywbem.Uint16,
                'NoSinglePointOfFailure' : SettingProvider.string_to_bool,
                'PackageRedundancyGoal' : pywbem.Uint16,
                'PackageRedundancyMax' : pywbem.Uint16,
                'PackageRedundancyMin' : pywbem.Uint16,
                'ParityLayout' : pywbem.Uint16,
        }

    @cmpi_logging.trace_method
    def get_setting_ignore(self, setting_provider):
        return {
                'CompressedElement': False,
                'CompressionRate': 1,
                'InitialSynchronization': 0,
                'SpaceLimit': 0,
                'ThinProvisionedInitialReserve': 0,
                'UseReplicationBuffer': 0,
        }

    @cmpi_logging.trace_method
    def get_setting_validators(self, setting_provider):
        return None


    @cmpi_logging.trace_method
    def do_delete_instance(self, device):
        LOG().info("DELETE LV: %s", device.path)
        action = blivet.deviceaction.ActionDestroyDevice(device)
        storage.do_storage_action(self.storage, [action])



# Copyright (C) 2012 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Jan Safranek <jsafrane@redhat.com>
# -*- coding: utf-8 -*-
"""
Module for LMI_VGStoragePool class.

LMI_VGStoragePool
-----------------

.. autoclass:: LMI_VGStoragePool
    :members:

"""

from lmi.storage.DeviceProvider import DeviceProvider
import pywbem
import blivet
import lmi.providers.cmpi_logging as cmpi_logging
from lmi.storage.SettingHelper import SettingHelper
from lmi.storage.SettingManager import StorageSetting
import lmi.storage.util.units as units
import lmi.storage.util.storage as storage
import math
from lmi.storage.SettingProvider import SettingProvider

LOG = cmpi_logging.get_logger(__name__)

class LMI_VGStoragePool(DeviceProvider, SettingHelper):
    """
        Provider of LMI_VGStoragePool.
    """
    @cmpi_logging.trace_method
    def __init__(self, *args, **kwargs):
        super(LMI_VGStoragePool, self).__init__(
                setting_classname='LMI_LVStorageSetting',
                *args, **kwargs)

    @cmpi_logging.trace_method
    def provides_name(self, object_name):
        """
            Returns True, if this class is provider for given CIM InstanceName.
        """
        if not object_name.has_key('InstanceID'):
            return False

        instance_id = object_name['InstanceID']
        if instance_id is None:
            return False
        parts = instance_id.split(":")
        if len(parts) != 3:
            return False
        if parts[0] != "LMI":
            return False
        if parts[1] != "VG":
            return False
        return True

    @cmpi_logging.trace_method
    def provides_device(self, device):
        """
            Returns True, if this class is provider for given Anaconda
            StorageDevice class.
        """
        if  isinstance(device, blivet.devices.LVMVolumeGroupDevice):
            return True
        return False

    @cmpi_logging.trace_method
    def get_device_for_name(self, object_name):
        """
            Returns Anaconda StorageDevice for given CIM InstanceName or
            None if no device is found.
        """
        if self.provides_name(object_name):
            instance_id = object_name['InstanceID']
            parts = instance_id.split(":")
            vgname = parts[2]
            for vg in self.storage.vgs:
                if vg.name == vgname:
                    return vg
            return None

    @cmpi_logging.trace_method
    def get_name_for_device(self, device):
        """
            Returns CIM InstanceName for given Anaconda StorageDevice.
            None if no device is found.
        """
        vgname = device.name
        name = pywbem.CIMInstanceName('LMI_VGStoragePool',
                namespace=self.config.namespace,
                keybindings={
                    'InstanceID' : "LMI:VG:" + vgname
                })
        return name

    @cmpi_logging.trace_method
    # pylint: disable-msg=W0221
    def get_instance(self, env, model, device=None):
        """
            Provider implementation of GetInstance intrinsic method.
            It fills all VGStoragePool properties.
        """
        if not self.provides_name(model):
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND, "Wrong keys.")
        if not device:
            device = self.get_device_for_name(model)
        if not device:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND,
                    "Cannot find the VG.")

        model['Primordial'] = False
        model['ElementName'] = device.name
        model['PoolID'] = device.name
        model['Name'] = device.path

        model['TotalManagedSpace'] = pywbem.Uint64(
                device.extents * device.peSize * units.MEGABYTE)
        model['RemainingManagedSpace'] = pywbem.Uint64(
                device.peFree * device.peSize * units.MEGABYTE)

        model['ExtentSize'] = pywbem.Uint64(device.peSize * units.MEGABYTE)
        model['TotalExtents'] = pywbem.Uint64(device.extents)
        model['RemainingExtents'] = pywbem.Uint64(device.peFree)
        model['UUID'] = device.uuid

        return model

    @cmpi_logging.trace_method
    def enum_instances(self, env, model, keys_only):
        """Enumerate instances.

        The WBEM operations EnumerateInstances and EnumerateInstanceNames
        are both mapped to this method.
        This method is a python generator

        Keyword arguments:
        env -- Provider Environment (pycimmb.ProviderEnvironment)
        model -- A template of the pywbem.CIMInstances to be generated.
            The properties of the model are already filtered according to
            the PropertyList from the request.  Only properties present in
            the model need to be given values.  If you prefer, you can
            always set all of the values, and the instance will be filtered
            for you.
        keys_only -- A boolean.  True if only the key properties should be
            set on the generated instances.

        Possible Errors:
        CIM_ERR_FAILED (some other unspecified error occurred)

        """
        model.path.update({'InstanceID': None})

        for device in self.storage.vgs:
            name = self.get_name_for_device(device)
            model['InstanceID'] = name['InstanceID']
            if keys_only:
                yield model
            else:
                yield self.get_instance(env, model, device)

    @cmpi_logging.trace_method
    def cim_method_getsupportedsizes(self, env, object_name,
                                     param_elementtype=None,
                                     param_goal=None,
                                     param_sizes=None):
        """Implements LMI_VGStoragePool.GetSupportedSizes() """
        rval = self.Values.GetSupportedSizes.Use_GetSupportedSizes_instead
        return (rval, [])

    @cmpi_logging.trace_method
    def cim_method_getsupportedsizerange(self, env, object_name,
                                         param_minimumvolumesize=None,
                                         param_maximumvolumesize=None,
                                         param_elementtype=None,
                                         param_volumesizedivisor=None,
                                         param_goal=None):
        """Implements LMI_VGStoragePool.GetSupportedSizeRange()
        param_minimumvolumesize --  The input parameter MinimumVolumeSize (type pywbem.Uint64)
            The minimum size for a volume/pool in bytes.

        param_maximumvolumesize --  The input parameter MaximumVolumeSize (type pywbem.Uint64)
            The maximum size for a volume/pool in bytes.

        param_elementtype --  The input parameter ElementType (type pywbem.Uint16 self.Values.GetSupportedSizeRange.ElementType)
            The type of element for which supported size ranges are
            reported. The Thin Provision values are only supported when
            the Thin Provisioning Profile is supported; the resulting
            StorageVolues/LogicalDisk shall have ThinlyPprovisioned set to
            true.

        param_volumesizedivisor --  The input parameter VolumeSizeDivisor (type pywbem.Uint64)
            A volume/pool size must be a multiple of this value which is
            specified in bytes.

        param_goal --  The input parameter Goal (type REF (pywbem.CIMInstanceName(classname='CIM_StorageSetting', ...))
            The StorageSetting for which supported size ranges should be
            reported for.

        Returns a two-tuple containing the return value (type pywbem.Uint32 self.Values.GetSupportedSizeRange)
        and a list of CIMParameter objects representing the output parameters

        Output parameters:
        MinimumVolumeSize -- (type pywbem.Uint64)
            The minimum size for a volume/pool in bytes.

        MaximumVolumeSize -- (type pywbem.Uint64)
            The maximum size for a volume/pool in bytes.

        VolumeSizeDivisor -- (type pywbem.Uint64)
            A volume/pool size must be a multiple of this value which is
            specified in bytes.

        Possible Errors:
        CIM_ERR_ACCESS_DENIED
        CIM_ERR_INVALID_PARAMETER (including missing, duplicate,
            unrecognized or otherwise incorrect parameters)
        CIM_ERR_NOT_FOUND (the target CIM Class or instance does not
            exist in the specified namespace)
        CIM_ERR_METHOD_NOT_AVAILABLE (the CIM Server is unable to honor
            the invocation request)
        CIM_ERR_FAILED (some other unspecified error occurred)

        """
        if not self.provides_name(object_name):
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND, "Wrong keys.")
        device = self.get_device_for_name(object_name)
        if not device:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND,
                    "Cannot find the VG.")

        # we support only logical disks for now (should be StorageExtent)
        etypes = self.Values.GetSupportedSizeRange.ElementType
        if (param_elementtype
                and param_elementtype != etypes.Logical_Disk):
            ret = self.Values.GetSupportedSizeRange.Invalid_Element_Type
            return (ret, [])

        # TODO: check Goal setting!

        extent_size = long(device.peSize * units.MEGABYTE)
        available_size = long(
                device.peSize * device.freeExtents * units.MEGABYTE)

        out_params = []
        out_params += [pywbem.CIMParameter('minimumvolumesize', type='uint64',
                           value=pywbem.Uint64(extent_size))]
        out_params += [pywbem.CIMParameter('maximumvolumesize', type='uint64',
                           value=pywbem.Uint64(available_size))]
        out_params += [pywbem.CIMParameter('volumesizedivisor', type='uint64',
                           value=pywbem.Uint64(extent_size))]
        rval = pywbem.Uint32(
            self.Values.GetSupportedSizeRange.Method_completed_OK)
        return (rval, out_params)

    @cmpi_logging.trace_method
    def _get_setting_for_device(self, device, setting_provider):
        """ Return setting for given device """
        _id = storage.get_persistent_name(device)
        setting = self.setting_manager.create_setting(
                self.setting_classname,
                StorageSetting.TYPE_CONFIGURATION,
                setting_provider.create_setting_id(_id),
                class_to_create=StorageSetting)
        setting.set_setting(self.get_redundancy(device))
        setting['ExtentSize'] = device.peSize * units.MEGABYTE
        setting['ElementName'] = device.path
        return setting

    @cmpi_logging.trace_method
    def enumerate_settings(self, setting_provider):
        """
            This method returns iterable with all instances of LMI_*Setting
            as Setting instances.
        """
        for lv in self.storage.vgs:
            yield self._get_setting_for_device(lv, setting_provider)

    @cmpi_logging.trace_method
    def get_setting_for_id(self, setting_provider, instance_id):
        """
            Return Setting instance, which corresponds to LMI_*Setting with
            given InstanceID.
            Return None if there is no such instance.

            Subclasses must override this method.
        """
        path = setting_provider.parse_setting_id(instance_id)
        if not path:
            return None
        device = storage.get_device_for_persistent_name(self.storage, path)
        if not path:
            return None
        if not isinstance(device,
                blivet.devices.LVMVolumeGroupDevice):
            LOG().trace_warn("InstanceID %s is not LVMLogicalVolumeDevice",
                    instance_id)
            return None
        return self._get_setting_for_device(device, setting_provider)

    @cmpi_logging.trace_method
    def get_associated_element_name(self, setting_provider, instance_id):
        """
            Return CIMInstanceName of ManagedElement for ElementSettingData
            association for setting with given ID.
            Return None if no such ManagedElement exists.
        """
        path = setting_provider.parse_setting_id(instance_id)
        if not path:
            return None
        device = storage.get_device_for_persistent_name(self.storage, path)
        if not device:
            return None
        if not isinstance(device,
                blivet.devices.LVMVolumeGroupDevice):
            LOG().trace_warn("InstanceID %s is not LVMLogicalVolumeDevice",
                    instance_id)
            return None
        return self.get_name_for_device(device)

    @cmpi_logging.trace_method
    def get_supported_setting_properties(self, setting_provider):
        """
            Return hash property_name -> constructor.
                constructor is a function which takes string argument
                and returns CIM value. (i.e. pywbem.Uint16
                or bool or string etc).
            This hash will be passed to SettingProvider.__init__
        """
        return {
                'ExtentSize': pywbem.Uint64,
                'DataRedundancyGoal': pywbem.Uint16,
                'DataRedundancyMax': pywbem.Uint16,
                'DataRedundancyMin': pywbem.Uint16,
                'ExtentStripeLength' : pywbem.Uint16,
                'ExtentStripeLengthMax' : pywbem.Uint16,
                'ExtentStripeLengthMin' : pywbem.Uint16,
                'NoSinglePointOfFailure' : SettingProvider.string_to_bool,
                'PackageRedundancyGoal' : pywbem.Uint16,
                'PackageRedundancyMax' : pywbem.Uint16,
                'PackageRedundancyMin' : pywbem.Uint16,
                'ParityLayout' : pywbem.Uint16,
        }

    @cmpi_logging.trace_method
    def get_setting_validators(self, setting_provider):
        return {
                'ExtentSize': self._check_extent_size
        }

    @cmpi_logging.trace_method
    def get_setting_ignore(self, setting_provider):
        return {
                'CompressedElement': False,
                'CompressionRate': 1,
                'InitialSynchronization': 0,
                'SpaceLimit': 0,
                'ThinProvisionedInitialReserve': 0,
                'UseReplicationBuffer': 0,
        }

    @cmpi_logging.trace_method
    def _check_extent_size(self, value):
        """
            Check if the given value is acceptable as
            VGStorageSetting.ExtentSize.
        """
        # lowest value is 1MB
        if value < units.MEGABYTE:
            raise pywbem.CIMError(pywbem.CIM_ERR_FAILED,
                    "Property ExtentSize must be at least 1MiB")
        # must be power of 2
        exp = math.log(value, 2)
        if math.floor(exp) != exp:
            raise pywbem.CIMError(pywbem.CIM_ERR_FAILED,
                    "Property ExtentSize must be power of 2")
        return True


    @cmpi_logging.trace_method
    def do_delete_instance(self, device):
        storage.log_storage_call("DELETE VG",
                {'device': device})
        action = blivet.deviceaction.ActionDestroyDevice(device)
        storage.do_storage_action(self.storage, [action])


    class Values(DeviceProvider.Values):
        class GetSupportedSizeRange(object):
            Method_completed_OK = pywbem.Uint32(0)
            Method_not_supported = pywbem.Uint32(1)
            Use_GetSupportedSizes_instead = pywbem.Uint32(2)
            Invalid_Element_Type = pywbem.Uint32(3)
            class ElementType(object):
                Storage_Pool = pywbem.Uint16(2)
                Storage_Volume = pywbem.Uint16(3)
                Logical_Disk = pywbem.Uint16(4)
                Thin_Provisioned_Volume = pywbem.Uint16(5)
                Thin_Provisioned_Logical_Disk = pywbem.Uint16(6)

        class GetSupportedSizes(object):
            Method_completed_OK = pywbem.Uint32(0)
            Method_not_supported = pywbem.Uint32(1)
            Use_GetSupportedSizes_instead = pywbem.Uint32(2)
            Invalid_Element_Type = pywbem.Uint32(3)
            class ElementType(object):
                Storage_Pool = pywbem.Uint16(2)
                Storage_Volume = pywbem.Uint16(3)
                Logical_Disk = pywbem.Uint16(4)
                Thin_Provisioned_Volume = pywbem.Uint16(5)
                Thin_Provisioned_Logical_Disk = pywbem.Uint16(6)



# Copyright (C) 2012-2013 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Jan Safranek <jsafrane@redhat.com>
"""
    Support functions for Blivet.
"""

import subprocess
import os
import parted
import pywbem
import blivet
import lmi.providers.cmpi_logging as cmpi_logging

GPT_TABLE_SIZE = 34 * 2  # there are two copies
MBR_TABLE_SIZE = 1

LOG = cmpi_logging.get_logger(__name__)

def _align_up(address, alignment):
    """ Align address to nearest higher address divisible by alignment."""
    return (address / alignment + 1) * alignment

def _align_down(address, alignment):
    """ Align address to nearest lower address divisible by alignment."""
    return (address / alignment) * alignment

@cmpi_logging.trace_function
def get_logical_partition_start(partition):
    """
        Return starting sector of logical partition metadata, relative to
        extended partition start.
    """
    disk = partition.disk
    ext = disk.format.partedDisk.getExtendedPartition()
    if not ext:
        raise pywbem.CIMError(pywbem.CIM_ERR_FAILED,
                'Cannot find extended partition.')

    metadata = None
    part = ext.nextPartition()
    while part is not None:
        if (part.type & parted.PARTITION_LOGICAL
                and part.type & parted.PARTITION_METADATA):
            metadata = part
        if part.path == partition.path:
            break
        part = part.nextPartition()

    if not part:
        raise pywbem.CIMError(pywbem.CIM_ERR_FAILED,
                'Cannot find the partition on the disk.')
    if not metadata:
        raise pywbem.CIMError(pywbem.CIM_ERR_FAILED,
                'Cannot find metadata for the partition.')

    return metadata.geometry.start

@cmpi_logging.trace_function
def get_partition_table_size(device):
    """
        Return size of partition table (in blocks) for given Anaconda
        StorageDevice instance.
    """
    if device.format:
        fmt = device.format
        if fmt.labelType == "gpt":
            return GPT_TABLE_SIZE * 2
        if fmt.labelType == "msdos":
            return MBR_TABLE_SIZE
    return 0

@cmpi_logging.trace_function
def get_available_sectors(device):
    """
        Return (start, end), where start is the first usable sector after
        partition table and end is the last usable sector before any
        partition table copy
    """
    size = device.partedDevice.length
    if device.format:
        fmt = device.format
        alignment = device.partedDevice.optimumAlignment.grainSize
        if fmt.labelType == "gpt":
            return (
                    _align_up(GPT_TABLE_SIZE, alignment),
                    _align_down(size - GPT_TABLE_SIZE - 1, alignment))
        if fmt.labelType == "msdos":
            return(
                    _align_up(MBR_TABLE_SIZE, alignment),
                    _align_down(size - 1, alignment))

    if isinstance(device, blivet.devices.PartitionDevice):
        if device.isExtended:
            return(
                    _align_up(0, alignment),
                    _align_down(size - 1, alignment))

    return (0, size - 1)

@cmpi_logging.trace_function
def remove_partition(storage, device):
    """
        Remove PartitionDevice from system, i.e. delete a partition.
    """
    action = blivet.deviceaction.ActionDestroyDevice(device)
    do_storage_action(storage, [action])

@cmpi_logging.trace_function
def do_storage_action(storage, actions):
    """
        Perform array Anaconda DeviceActions on given Storage instance.
    """
    do_partitioning = False
    need_reset = False

    for action in actions:
        LOG().trace_info("Running action " + str(action))
        LOG().trace_info("    on device " + repr(action.device))

        if (isinstance(action.device, blivet.devices.PartitionDevice)
                and isinstance(action,
                        blivet.deviceaction.ActionCreateDevice)):
            do_partitioning = True

        if (isinstance(action.device, blivet.devices.PartitionDevice) and not
                isinstance(action, blivet.deviceaction.ActionDestroyFormat)):
            # The action is partition manipulation action, Blivet needs reload
            # to get parted devices right
            need_reset = True
        elif not isinstance(action,
                    (blivet.deviceaction.ActionDestroyFormat,
                    blivet.deviceaction.ActionDestroyDevice)):
            # The action is device creation/manipulation, Blivet needs reload
            # to get new/changed device properties (like udev symlinks,
            # uuid etc.)
            need_reset = True

        storage.devicetree.registerAction(action)
    try:
        if do_partitioning:
            # this must be called when creating a partition
            LOG().trace_verbose("Running doPartitioning()")
            blivet.partitioning.doPartitioning(storage=storage)

        storage.devicetree.processActions(dryRun=False)

        for action in actions:
            if not isinstance(action,
                    blivet.deviceaction.ActionDestroyDevice):
                LOG().trace_verbose(
                        "Result: " + repr(action.device))

    finally:
        if need_reset:
            os.system('udevadm trigger --subsystem-match block')
            os.system('udevadm settle')
            storage.reset()

def log_storage_call(msg, args):
    """
        Log a storage action to log.
        INFO level will be used. The message should have format
        'CREATE <type>' or 'DELETE <type>', where <type> is type of device
        (PARTITION, MDRAID, VG, LV, ...). The arguments will be printed
        after it in no special orded, only StorageDevice instances
        will be replaced with device.path.
    """
    print_args = {}
    for (key, value) in args.iteritems():
        if isinstance(value, blivet.devices.StorageDevice):
            value = value.path
        if key == 'parents':
            value = [d.path for d in value]
        print_args[key] = value

    LOG().info(msg + ": " + str(print_args))

@cmpi_logging.trace_function
def get_persistent_name(device):
    """
    Return stable device name of given device.

    Current order of significance: by-id, by-partuuid, by-path, by-uuid, /dev

    :param device: (``StorageDevice``) The device.
    :returns: ``string``
    """
    prefixes = ["/dev/disk/by-id/", "/dev/disk/by-partuuid/",
            "/dev/disk/by-path/", "/dev/disk/by-uuid/"]
    links = device.deviceLinks
    found = False
    # Find a symlink which matches the topmost prefix"
    for prefix in prefixes:
        for link in links:
            if link.startswith(prefix):
                found = True
                break
        if found:
            break
    if found:
        return link
    return device.path

@cmpi_logging.trace_function
def get_device_for_persistent_name(_blivet, name):
    """
    Return device for given device name or device symlink name as returned
    by get_persistent_name.

    :param _blivet: (``Blivet``) Blivet instance.
    :param name: (``string``) Name of the device or device symlink.
    :returns: ``StorageDevice`` or ``None``, if no device matches the name.
    """
    for device in _blivet.devices:
        if name == device.path:
            return device
        if name in device.deviceLinks:
            return device
    return None

@cmpi_logging.trace_function
def check_empty_parameters(**kwargs):
    """
    Check for empty parameters. Throw an error if there are any.
    """
    for p, a in kwargs.items():
        if a is None:
            raise pywbem.CIMError(pywbem.CIM_ERR_INVALID_PARAMETER,
                                  "Empty parameter: %s." % p)

@cmpi_logging.trace_function
def assert_unused(_blivet, devices):
    """
    Check that given devices are not used. Throw pywbem.CIMError if so.

    'Used' means that the device is either mounted or other device depends on
    it.

    :param _blivet: (``Blivet``) Blivet instance.
    :param devices: (array of ``StorageDevice``s) Devices to check.
    :returns None:
    """
    for device in devices:
        fs = device.format
        if fs:
            try:
                mountpoint = fs.mountpoint
            except AttributeError:
                mountpoint = None
            if mountpoint:
                # The device has mounted filesystem -> it is used.
                raise pywbem.CIMError(pywbem.CIM_ERR_FAILED,
                    "Device %s is mounted at %s." % (device.path, mountpoint))

        children = _blivet.deviceDeps(device)
        for child in children:
            # Members of stopped stopped MD RAID and other stopped devices
            # are unused.
            if not child.status:
                continue

            # Unmounted BTRFs formats are unused.
            if isinstance(child, blivet.devices.BTRFSVolumeDevice):
                assert_unused(_blivet, [child])
                continue

            raise pywbem.CIMError(pywbem.CIM_ERR_FAILED,
                "Device %s is used by %s." % (device.path, child.path))



# Copyright (C) 2012 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Jan Safranek <jsafrane@redhat.com>
# -*- coding: utf-8 -*-

from lmi.storage.StorageConfiguration import StorageConfiguration
import lmi.providers.cmpi_logging as cmpi_logging
import unittest

import sys
import StringIO

class CMPILoggerMock(object):
    """ Mockup of cmpi_bindings_pywbem Logger and env."""
    def __init__(self, stream):
        self.stream = stream

    def get_logger(self):
        return self

    def log_error(self, msg):
        self.stream.write("E:" + msg)

    def log_warn(self, msg):
        self.stream.write("W:" + msg)

    def log_info(self, msg):
        self.stream.write("I:" + msg)

    def log_debug(self, msg):
        self.stream.write("D:" + msg)

    def trace_warn(self, component, msg):
        self.stream.write("w:" + msg)

    def trace_info(self, component, msg):
        self.stream.write("i:" + msg)

    def trace_verbose(self, component, msg):
        self.stream.write("v:" + msg)

class TestLogging(unittest.TestCase):
    """
        Test logging with various configurations + also with dynamic
        configuration updates.
        
        This test is very fragile, it depends on trace messages in LogManager
        and StorageConfiguration.
    """
    def setUp(self):
        self.cmpi_stream = StringIO.StringIO()
        self.stderr_stream = StringIO.StringIO()
        # override stderr
        self.oldstderr = sys.stderr
        sys.stderr = self.stderr_stream

        self.cmpi_logger = CMPILoggerMock(self.cmpi_stream)
        self.logmgr = cmpi_logging.LogManager(self.cmpi_logger)
        self.logger = cmpi_logging.logger

    def test_default(self):
        """
            Test default configuration
            - INFO goes to CMPI.
            - nothing goes to stderr
        """
        StorageConfiguration.CONFIG_PATH = "/"
        StorageConfiguration.CONFIG_FILE = "/configs/not-existing.conf"
        cfg = StorageConfiguration()
        self.logmgr.set_config(cfg)

        # log at various log levels)
        self.logger.error("E1")
        self.logger.warn("W1")
        self.logger.info("I1")
        self.logger.debug("D1")
        self.logger.trace_warn("w1")
        self.logger.trace_info("i1")
        self.logger.trace_verbose("v1")

        # check that the messages went through CMPILoggerMock
        cmpi = self.cmpi_stream.getvalue()
        stderr = self.stderr_stream.getvalue()
        self.assertEqual(cmpi, "I:INFO: CMPI log startedE:ERROR: E1W:WARNING: W1I:INFO: I1")
        self.assertEqual(stderr, "")

    def test_tracing(self):
        """
            Test with enabled tracing
            - DEBUG and TRACE goes to CMPI.
            - nothing goes to stderr
        """
        StorageConfiguration.CONFIG_PATH = "/"
        StorageConfiguration.CONFIG_FILE = "/configs/not-existing.conf"
        cfg = StorageConfiguration()
        cfg.config.set("Log", "Debug", "true")
        self.logmgr.set_config(cfg)

        # log at various log levels)
        self.logger.error("E1")
        self.logger.warn("W1")
        self.logger.info("I1")
        self.logger.debug("D1")
        self.logger.trace_warn("w1")
        self.logger.trace_info("i1")
        self.logger.trace_verbose("v1")

        # check that the messages went through CMPILoggerMock
        cmpi = self.cmpi_stream.getvalue()
        stderr = self.stderr_stream.getvalue()
        self.assertEqual(cmpi, "I:INFO: CMPI log startedv:DEBUG: Exiting LogManager._config_changedv:DEBUG: Exiting LogManager.set_configE:ERROR: E1W:WARNING: W1I:INFO: I1v:DEBUG: D1w:Level 19: w1i:Level 18: i1v:DEBUG: v1")
        self.assertEqual(stderr, "")

    def test_stderr(self):
        """
            Test with enabled stderr
            - INFO goes to CMPI.
            - INFO goes to CMPI.
        """
        StorageConfiguration.CONFIG_PATH = "/"
        StorageConfiguration.CONFIG_FILE = "/configs/not-existing.conf"
        cfg = StorageConfiguration()
        cfg.config.set("Log", "Stderr", "true")

        self.logmgr.set_config(cfg)

        # log at various log levels)
        self.logger.error("E1")
        self.logger.warn("W1")
        self.logger.info("I1")
        self.logger.debug("D1")
        self.logger.trace_warn("w1")
        self.logger.trace_info("i1")
        self.logger.trace_verbose("v1")

        # check that the messages went through CMPILoggerMock
        cmpi = self.cmpi_stream.getvalue()
        stderr = self.stderr_stream.getvalue()
        self.assertEqual(cmpi, "I:INFO: CMPI log startedI:INFO: Started logging to stderr.E:ERROR: E1W:WARNING: W1I:INFO: I1")
        self.assertEqual(stderr, "INFO: Started logging to stderr.\nERROR: E1\nWARNING: W1\nINFO: I1\n")

    def test_tracing_stderr(self):
        """
            Test with enabled tracing and stderr
            - DEBUG and TRACE goes to CMPI.
            - DEBUG and TRACE goes to stderr.
        """
        StorageConfiguration.CONFIG_PATH = "/"
        StorageConfiguration.CONFIG_FILE = "/configs/not-existing.conf"
        cfg = StorageConfiguration()
        cfg.config.set("Log", "Debug", "true")
        cfg.config.set("Log", "Stderr", "true")
        self.logmgr.set_config(cfg)

        # log at various log levels)
        self.logger.error("E1")
        self.logger.warn("W1")
        self.logger.info("I1")
        self.logger.debug("D1")
        self.logger.trace_warn("w1")
        self.logger.trace_info("i1")
        self.logger.trace_verbose("v1")

        # check that the messages went through CMPILoggerMock
        cmpi = self.cmpi_stream.getvalue()
        stderr = self.stderr_stream.getvalue()
        self.assertEqual(cmpi, "I:INFO: CMPI log startedI:INFO: Started logging to stderr.v:DEBUG: Exiting LogManager._config_changedv:DEBUG: Exiting LogManager.set_configE:ERROR: E1W:WARNING: W1I:INFO: I1v:DEBUG: D1w:Level 19: w1i:Level 18: i1v:DEBUG: v1")
        self.assertEqual(stderr, "INFO: Started logging to stderr.\nDEBUG: Exiting LogManager._config_changed\nDEBUG: Exiting LogManager.set_config\nERROR: E1\nWARNING: W1\nINFO: I1\nDEBUG: D1\nLevel 19: w1\nLevel 18: i1\nDEBUG: v1\n")

    def test_dynamic_change(self):
        """
            Test with changing configuration
        """
        # start with the default on
        StorageConfiguration.CONFIG_PATH = "/"
        StorageConfiguration.CONFIG_FILE = "/configs/not-existing.conf"
        cfg = StorageConfiguration()
        self.logmgr.set_config(cfg)

        # log at various log levels)
        self.logger.error("E1")
        self.logger.warn("W1")
        self.logger.info("I1")
        self.logger.debug("D1")
        self.logger.trace_warn("w1")
        self.logger.trace_info("i1")
        self.logger.trace_verbose("v1")

        # enable tracing and stderr
        cfg.config.set("Log", "Debug", "true")
        cfg.config.set("Log", "Stderr", "true")
        cfg._call_listeners()

        # log at various log levels)
        self.logger.error("E2")
        self.logger.warn("W2")
        self.logger.info("I2")
        self.logger.debug("D2")
        self.logger.trace_warn("w2")
        self.logger.trace_info("i2")
        self.logger.trace_verbose("v2")

        # disable tracing and stderr
        cfg.config.set("Log", "Debug", "false")
        cfg.config.set("Log", "Stderr", "false")
        cfg._call_listeners()

        # log at various log levels)
        self.logger.error("E3")
        self.logger.warn("W3")
        self.logger.info("I3")
        self.logger.debug("D3")
        self.logger.trace_warn("w3")
        self.logger.trace_info("i3")
        self.logger.trace_verbose("v3")

        # check that the messages went through CMPILoggerMock
        cmpi = self.cmpi_stream.getvalue()
        stderr = self.stderr_stream.getvalue()
        self.assertEqual(cmpi, "I:INFO: CMPI log startedE:ERROR: E1W:WARNING: W1I:INFO: I1I:INFO: Started logging to stderr.v:DEBUG: Exiting LogManager._config_changedv:DEBUG: Exiting StorageConfiguration._call_listenersE:ERROR: E2W:WARNING: W2I:INFO: I2v:DEBUG: D2w:Level 19: w2i:Level 18: i2v:DEBUG: v2v:DEBUG: Entering StorageConfiguration._call_listenersv:DEBUG: Entering LogManager._config_changedI:INFO: Stopped logging to stderr.E:ERROR: E3W:WARNING: W3I:INFO: I3")
        self.assertEqual(stderr, "INFO: Started logging to stderr.\nDEBUG: Exiting LogManager._config_changed\nDEBUG: Exiting StorageConfiguration._call_listeners\nERROR: E2\nWARNING: W2\nINFO: I2\nDEBUG: D2\nLevel 19: w2\nLevel 18: i2\nDEBUG: v2\nDEBUG: Entering StorageConfiguration._call_listeners\nDEBUG: Entering LogManager._config_changed\nINFO: Stopped logging to stderr.\n")

    def tearDown(self):
        self.logmgr.destroy()
        # restore old stderr
        sys.stderr = self.oldstderr
        # and print all messages to it, we want asserts there
        sys.stderr.write(self.stderr_stream.getvalue())


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/python
# -*- Coding:utf-8 -*-
#
# Copyright (C) 2012 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Jan Safranek <jsafrane@redhat.com>

from test_base import StorageTestBase
import pywbem
import unittest

class TestSettings(StorageTestBase):
    """
        Test common features of settings.
         - it can be cloned
         - it can be modified
         - it can be persisted to disk
         - it can be removed
    """
    @classmethod
    def setUpClass(cls):
        super(TestSettings, cls).setUpClass()

        # find an instance we will clone
        settings = cls.wbemconnection.EnumerateInstanceNames('LMI_DiskPartitionConfigurationSetting')        
        if len(settings) < 1:
            raise unittest.SkipTest("There is no LMI_DiskPartitionConfigurationSetting on system. Add a partition!")
        cls.master = settings[0]

    def create_clone(self):
        """ Create and return a clone of master. """
        (ret, outvars) = self.wbemconnection.InvokeMethod(
                'CloneSetting',
                self.master)
        # check return parameters
        self.assertTrue(ret == 0)
        clone_path = outvars['Clone']
        self.assertTrue(clone_path)
        clone = self.wbemconnection.GetInstance(clone_path)
        return clone
        
    def test_clone(self):
        """ Test that master.clone works as expected """
        clone = self.create_clone()
        self.assertTrue(clone)
        
        # check the clone is the same as the master
        master = self.wbemconnection.GetInstance(self.master)
        self.assertEqual(master['Bootable'], clone['Bootable'])
        self.assertEqual(master['Caption'], clone['Caption'])
        self.assertEqual(master['ConfigurationName'], clone['ConfigurationName'])
        self.assertEqual(master['Description'], clone['Description'])
        self.assertEqual(master['ElementName'], clone['ElementName'])
        self.assertEqual(master['Hidden'], clone['Hidden'])
        self.assertEqual(master['PartitionType'], clone['PartitionType'])
        # check it has different ID
        self.assertNotEqual(self.master['InstanceID'], clone['InstanceID'])
        # check the clone is transient
        self.assertEqual(1, clone['ChangeableType'])
        
    def test_modify_transient(self):
        """ Test changing of transient instance. """
        clone = self.create_clone()
        self.assertTrue(clone)
        clone_path = clone.path
        # change Bootable, ElementName and PartitionType to initial values
        # (we don't know what clone values are)
        clone['Bootable'] = False
        clone['PartitionType'] = pywbem.Uint16(0) # unknown
        clone['ElementName'] = 'myElementName'
        self.wbemconnection.ModifyInstance(clone)

        clone = self.wbemconnection.GetInstance(clone_path)
        self.assertTrue(clone)
        self.assertEqual(clone['Bootable'], False)
        self.assertEqual(clone['ElementName'], 'myElementName')
        self.assertEqual(clone['PartitionType'], 0)

        # change Bootable, ElementName and PartitionType again
        clone['Bootable'] = True
        clone['PartitionType'] = pywbem.Uint16(1)
        clone['ElementName'] = 'myElementName2'
        self.wbemconnection.ModifyInstance(clone)
        clone = self.wbemconnection.GetInstance(clone_path)
        self.assertTrue(clone)
        self.assertEqual(clone['Bootable'], True)
        self.assertEqual(clone['ElementName'], 'myElementName2')
        self.assertEqual(clone['PartitionType'], 1)

        # change ChangeableType
        # not changeable - persistent -> error
        clone['ChangeableType'] = pywbem.Uint16(0) 
        self.assertRaises(pywbem.CIMError, self.wbemconnection.ModifyInstance, clone)

        # not changeable - transient -> error
        clone['ChangeableType'] = pywbem.Uint16(4) 
        self.assertRaises(pywbem.CIMError, self.wbemconnection.ModifyInstance, clone)

        # changeable - transient -> nothing happens
        clone['ChangeableType'] = pywbem.Uint16(1) 
        self.wbemconnection.ModifyInstance(clone)

    def test_restart_transient(self):
        """ Test that CIMOM restart removes all transient. """
        clone = self.create_clone()
        self.assertTrue(clone)
        clone_path = clone.path
        
        self.restart_cim()
        
        self.assertRaises(pywbem.CIMError,
                          self.wbemconnection.GetInstance,
                          clone_path)

    def test_delete_transient(self):
        """ Test that transient config can be removed. """
        clone = self.create_clone()
        self.assertTrue(clone)
        clone_path = clone.path
        
        self.wbemconnection.DeleteInstance(clone_path)
        
        self.assertRaises(pywbem.CIMError,
                          self.wbemconnection.GetInstance,
                          clone_path)

    def test_delete_persistent(self):
        """ Test that persistent config can be removed. """
        clone = self.create_clone()
        self.assertTrue(clone)
        clone_path = clone.path

        # changeable - persistent -> clone is stored
        clone['ChangeableType'] = pywbem.Uint16(2)
        self.wbemconnection.ModifyInstance(clone)
        
        self.wbemconnection.DeleteInstance(clone_path)        
        self.assertRaises(pywbem.CIMError,
                          self.wbemconnection.GetInstance,
                          clone_path)
                          
        # check the instance is gone also after restart
        self.restart_cim()
        self.assertRaises(pywbem.CIMError,
                          self.wbemconnection.GetInstance,
                          clone_path)
        
        
        
    def test_modify_persistent(self):
        """ 
            Test that persistent setting can be modified and the changes
            survive CIMOM restart.
        """
        clone = self.create_clone()
        self.assertTrue(clone)
        clone_path = clone.path
        # change Bootable, ElementName and PartitionType to initial values
        # (we don't know what clone values are)
        clone['Bootable'] = False
        clone['PartitionType'] = pywbem.Uint16(0) # unknown
        clone['ElementName'] = 'myElementName'
        
        self.wbemconnection.ModifyInstance(clone)
        # changeable - persistent -> clone is stored
        clone['ChangeableType'] = pywbem.Uint16(2)
        self.wbemconnection.ModifyInstance(clone)
        
        # restart the service and check the clone survived
        self.restart_cim()
        clone = self.wbemconnection.GetInstance(clone_path)
        self.assertTrue(clone)
        self.assertEqual(clone['Bootable'], False)
        self.assertEqual(clone['ElementName'], 'myElementName')
        self.assertEqual(clone['PartitionType'], 0)
        self.assertEqual(clone['ChangeableType'], 2)
        
        # try to change the clone
        clone['Bootable'] = True
        clone['PartitionType'] = pywbem.Uint16(1)
        clone['ElementName'] = 'myElementName1'
        self.wbemconnection.ModifyInstance(clone)

        clone = self.wbemconnection.GetInstance(clone_path)
        self.assertTrue(clone)
        self.assertEqual(clone['Bootable'], True)
        self.assertEqual(clone['ElementName'], 'myElementName1')
        self.assertEqual(clone['PartitionType'], 1)
        self.assertEqual(clone['ChangeableType'], 2)

        # change ChangeableType
        # not changeable - persistent -> error
        clone['ChangeableType'] = pywbem.Uint16(0) 
        self.assertRaises(pywbem.CIMError, self.wbemconnection.ModifyInstance, clone)

        # not changeable - transient -> error
        clone['ChangeableType'] = pywbem.Uint16(4) 
        self.assertRaises(pywbem.CIMError, self.wbemconnection.ModifyInstance, clone)

        # changeable - transient -> error
        clone['ChangeableType'] = pywbem.Uint16(1) 
        self.assertRaises(pywbem.CIMError, self.wbemconnection.ModifyInstance, clone)
        
        # changeable - persistent -> nothing happens
        clone['ChangeableType'] = pywbem.Uint16(2)
        self.wbemconnection.ModifyInstance(clone)
        
        # restart the service and check the clone survived
        self.restart_cim()
        clone = self.wbemconnection.GetInstance(clone_path)
        self.assertTrue(clone)
        self.assertEqual(clone['Bootable'], True)
        self.assertEqual(clone['ElementName'], 'myElementName1')
        self.assertEqual(clone['PartitionType'], 1)
        self.assertEqual(clone['ChangeableType'], 2)
        

if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2012 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Jan Safranek <jsafrane@redhat.com>
# -*- coding: utf-8 -*-
"""
Module for LMI_InstalledPartitionTable class.

LMI_InstalledPartitionTable
---------------------------

.. autoclass:: LMI_InstalledPartitionTable
    :members:

"""

import blivet.formats
from lmi.storage.BaseProvider import BaseProvider
import pywbem
import lmi.providers.cmpi_logging as cmpi_logging

class LMI_InstalledPartitionTable(BaseProvider):
    """
        LMI_InstalledPartitionTable provider implementation.
    """

    @cmpi_logging.trace_method
    def __init__(self, *args, **kwargs):
        super(LMI_InstalledPartitionTable, self).__init__(*args, **kwargs)
        mgr = self.provider_manager
        self.capabilities_provider = mgr.get_capabilities_provider_for_class(
                "LMI_DiskPartitionConfigurationCapabilities")

    @cmpi_logging.trace_method
    def get_instance(self, env, model):
        """
            Provider implementation of GetInstance intrinsic method.
        """
        device = self.provider_manager.get_device_for_name(model['Antecedent'])
        if not device:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND,
                    "Cannot find Antecedent extent.")
        capabilities = self.capabilities_provider.get_capabilities_for_device(
                device)
        if not capabilities:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND,
                    "Cannot find capabilities of the Antecedent extent.")

        if model['Dependent']['InstanceID'] != capabilities['InstanceID']:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND,
                    "Capabilities are not associated to Antecedent extent.")

        return model

    @cmpi_logging.trace_method
    def get_capabilities_name_for_device(self, device):
        """
            Return CIMInstanceName of DiskPartitionConfigurationCapabilities
            for given device
            or None if there is no partition table on the device.
        """
        capabilities = self.capabilities_provider.get_capabilities_for_device(
                device)
        if capabilities:
            return self.capabilities_provider.get_name_for_id(
                    capabilities['InstanceID'])
        return None

    @cmpi_logging.trace_method
    def enum_instances(self, env, model, keys_only):
        """
            Provider implementation of EnumerateInstances intrinsic method.
        """
        model.path.update({'Dependent': None, 'Antecedent': None})

        for device in self.storage.devices:
            fmt_class = blivet.formats.disklabel.DiskLabel
            if device.format and isinstance(device.format, fmt_class):
                model['Antecedent'] = self.provider_manager.get_name_for_device(
                        device)
                model['Dependent'] = self.get_capabilities_name_for_device(
                        device)
                yield model


    @cmpi_logging.trace_method
    def references(self, env, object_name, model, result_class_name, role,
                   result_role, keys_only):
        """Instrument Associations."""
        return self.simple_references(env, object_name, model,
                result_class_name, role, result_role, keys_only,
                "CIM_StorageExtent",
                "CIM_DiskPartitionConfigurationCapabilities")



#!/usr/bin/python
# -*- Coding:utf-8 -*-
#
# Copyright (C) 2012 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Jan Safranek <jsafrane@redhat.com>

from test_base import StorageTestBase
import unittest
import pywbem



class TestVGPoolMethods(StorageTestBase):
    """
        Test TestVGPoolMethods methods.
    """

    VG_CLASS = "LMI_VGStoragePool"
    STYLE_EMBR = 4100
    STYLE_MBR = 2
    STYLE_GPT = 3
    PARTITION_CLASS = "LMI_GenericDiskPartition"


    def setUp(self):
        """ Find storage service. """
        super(TestVGPoolMethods, self).setUp()
        self.service = self.wbemconnection.EnumerateInstanceNames(
                "LMI_StorageConfigurationService")[0]
        self.part_service = self.wbemconnection.EnumerateInstanceNames(
                "LMI_DiskPartitionConfigurationService")[0]

        vgname = self._create_vg()
        self.vg = self.wbemconnection.GetInstance(vgname)


    def tearDown(self):
        self._destroy_vg(self.vg.path)
        super(TestVGPoolMethods, self).tearDown()


    def _create_vg(self):
        """
            Create a partition and Volume Group on it and return its
            CIMInstanceName.
        """
        (ret, outparams) = self.invoke_async_method(
                "CreateOrModifyVG",
                self.service,
                int, 'Pool',
                InExtents=self.partition_names[:1],
                ElementName='tstName')
        self.assertEqual(ret, 0)
        return outparams['Pool']

    def _destroy_vg(self, vgname):
        """ Destroy VG and its partition. """
        self.invoke_async_method("DeleteVG", self.service, int, None,
                Pool=vgname)

    def test_supported_sizes(self):
        """ Test GetSupportedSizes() """
        (retval, _) = self.wbemconnection.InvokeMethod(
                "GetSupportedSizes",
                self.vg.path)
        self.assertEqual(retval, 2)

    def test_supported_range_noparam(self):
        """ Test GetSupportedSizeRange without any parameters. """
        (retval, outparams) = self.wbemconnection.InvokeMethod(
                "GetSupportedSizeRange",
                self.vg.path)
        self.assertEqual(retval, 0)
        self.assertEqual(len(outparams), 3)

        self.assertEqual(
                outparams['minimumvolumesize'], self.vg['ExtentSize'])
        self.assertEqual(
                outparams['maximumvolumesize'],
                self.vg['RemainingExtents'] * self.vg['ExtentSize'])
        self.assertEqual(
                outparams['volumesizedivisor'], self.vg['ExtentSize'])

    def test_supported_range_elementtype(self):
        """ Test GetSupportedSizeRange with ElementType parameter. """
        (retval, outparams) = self.wbemconnection.InvokeMethod(
                "GetSupportedSizeRange",
                self.vg.path,
                ElementType=pywbem.Uint16(4))  # logical disk - OK

        self.assertEqual(retval, 0)
        self.assertEqual(len(outparams), 3)

        self.assertEqual(
                outparams['minimumvolumesize'], self.vg['ExtentSize'])
        self.assertEqual(
                outparams['maximumvolumesize'],
                self.vg['RemainingExtents'] * self.vg['ExtentSize'])
        self.assertEqual(
                outparams['volumesizedivisor'], self.vg['ExtentSize'])


        (retval, outparams) = self.wbemconnection.InvokeMethod(
                "GetSupportedSizeRange",
                self.vg.path,
                ElementType=pywbem.Uint16(2))  # storage pool - fail
        self.assertEqual(retval, 3)  # invalid element type

        (retval, outparams) = self.wbemconnection.InvokeMethod(
                "GetSupportedSizeRange",
                self.vg.path,
                ElementType=pywbem.Uint16(3))  # storage volume - fail
        self.assertEqual(retval, 3)  # invalid element type

        (retval, outparams) = self.wbemconnection.InvokeMethod(
                "GetSupportedSizeRange",
                self.vg.path,
                ElementType=pywbem.Uint16(5))  # thin storage volume - fail
        self.assertEqual(retval, 3)  # invalid element type

        (retval, outparams) = self.wbemconnection.InvokeMethod(
                "GetSupportedSizeRange",
                self.vg.path,
                ElementType=pywbem.Uint16(6))  # thin logical disk - fail
        self.assertEqual(retval, 3)  # invalid element type

        # TODO: check goal

if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2013 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Jan Safranek <jsafrane@redhat.com>
# -*- coding: utf-8 -*-
"""
Module for LMI_LocalFilesystem.

LMI_LocalFileSystem
-------------------

.. autoclass:: LMI_LocalFileSystem
    :members:

"""

from lmi.storage.LocalFileSystemProvider import LocalFileSystemProvider
import lmi.providers.cmpi_logging as cmpi_logging
import blivet.formats

LOG = cmpi_logging.get_logger(__name__)

class LMI_LocalFileSystem(LocalFileSystemProvider):
    """
        Generic file system provider for filesystems which do not have
        it's own provider.
    """
    @cmpi_logging.trace_method
    def __init__(self, *args, **kwargs):
        super(LMI_LocalFileSystem, self).__init__(
                classname="LMI_LocalFileSystem",
                device_type=None,
                setting_classname="LMI_FileSystemSetting",
                *args, **kwargs)

    @cmpi_logging.trace_method
    def provides_format(self, device, fmt):
        if fmt is None:
            return False

        # Skip non-devices (e.g. for tmpfs, sysfs, ...)
        if  isinstance(device, blivet.devices.NoDevice):
            return False

        # skip all non-filesystems
        if not isinstance(fmt, blivet.formats.fs.FS):
            return False

        # TODO: implement btrfs subvolumes
        if isinstance(device, blivet.devices.BTRFSSubVolumeDevice):
            return False

        # For BTRFS, we show LMI_LocalFileSystem for each BTRFSVolumeDevice
        if (isinstance(fmt, blivet.formats.fs.BTRFS)
                and not isinstance(device, blivet.devices.BTRFSVolumeDevice)):
            # This is not BTRFSVolumeDevice
            return False

        # TODO: skip formats with its own classes (currently none)

        # skip 'Unknown' format
        if fmt.type is None:
            return False
        return True

    def get_devices(self, device, fmt):
        if isinstance(fmt, blivet.formats.fs.BTRFS):
            if isinstance(device, blivet.devices.BTRFSSubVolumeDevice):
                # TODO: implement subvolumes
                return []
            if not isinstance(device, blivet.devices.BTRFSVolumeDevice):
                # We have real block device. Find the BTRFSVolume device for
                # it - it must be its only child.
                children = self.storage.devicetree.getChildren(device)
                if len(children) != 1:
                    LOG().trace_warn("Failed to find btrfs volume"
                            " device on %s: %s", str(device), str(children))
                    return []
                device = children[0]
            if isinstance(device, blivet.devices.BTRFSVolumeDevice):
                return device.parents
            else:
                LOG().trace_warn("Failed to find btrfs volume device on %s",
                        str(device))
                return []
        return [device]

    @cmpi_logging.trace_method
    def get_uuid(self, device, fmt):
        # We need to override it for BTRFS, it's uuid is in BTRFSVolumeDevice,
        # not it format
        if (isinstance(fmt, blivet.formats.fs.BTRFS)
            and isinstance(device, blivet.devices.BTRFSVolumeDevice)):
            uuid = device.uuid
            if uuid:
                return uuid
        return super(LMI_LocalFileSystem, self).get_uuid(device, fmt)



# Copyright (C) 2013 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Jan Safranek <jsafrane@redhat.com>
# -*- coding: utf-8 -*-
"""
Module for LMI_LocalFilesystem.

LMI_TransientFileSystem
-----------------------

.. autoclass:: LMI_TransientFileSystem
    :members:

"""

from lmi.storage.LocalFileSystemProvider import LocalFileSystemProvider
import lmi.providers.cmpi_logging as cmpi_logging
import blivet.formats
import pywbem
import os

class LMI_TransientFileSystem(LocalFileSystemProvider):
    """
        Generic file system provider for filesystems which do not have
        it's own provider and device.
    """
    @cmpi_logging.trace_method
    def __init__(self, *args, **kwargs):
        super(LMI_TransientFileSystem, self).__init__(
                classname="LMI_TransientFileSystem",
                device_type=None,
                setting_classname=None,
                *args, **kwargs)

    @cmpi_logging.trace_method
    def provides_format(self, device, fmt):
        if fmt is None:
            return False

        # Skip all non-filesystems
        if not isinstance(fmt, blivet.formats.fs.FS):
            return False

        # Skip 'Unknown' format
        if fmt.type is None:
            return False

        # Skip not mounted filesystems, there is no unique ID which we can
        # use to reference them.
        if fmt.mountpoint is None:
            return False

        if  isinstance(device, blivet.devices.NoDevice):
            return True

        return False

    def get_instance(self, env, model, fmt=None):
        model = super(LMI_TransientFileSystem, self).get_instance(env, model, fmt)

        if not fmt:
            (_device, fmt) = self.get_format_for_name(model)
        else:
            _device = self.storage.devicetree.getDeviceByPath(fmt.device)
        if not fmt:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND,
                    "Cannot find the format.")

        model['PersistenceType'] = self.Values.PersistenceType.Temporary
        return model

    def get_format_id(self, device, fmt):
        return "PATH=" + fmt.mountpoint

    def get_format_for_id(self, name):
        if name.startswith("PATH="):
            (_unused, mountpoint) = name.split("=")
            for (path, device) in self.storage.mountpoints.iteritems():
                if path == mountpoint:
                    return (device, device.format)
        return (None, None)



#!/usr/bin/python
# -*- Coding:utf-8 -*-
#
# Copyright (C) 2012 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Jan Safranek <jsafrane@redhat.com>
#

""" Base module for all storage tests. """

import os
import unittest
import pywbem
import subprocess
import pyudev
import time
import socket
import Queue
import threading

class StorageTestBase(unittest.TestCase):
    """
        This is base class of OpenLMI storage tests.
        It monitors all devices created between setUp and tearDown
        and tries to remove them in tearDown.
    """

    SYSTEM_CLASS_NAME = "PG_ComputerSystem"
    DISK_CLASS = "LMI_StorageExtent"

    DEFAULT_JOB_TIMEOUT = 5  # seconds

    JOB_CREATED = 4096  # usual value of Method Parameters Checked - Job Started

    @classmethod
    def setUpClass(cls):
        cls.url = os.environ.get("LMI_CIMOM_URL", "https://localhost:5989")
        cls.username = os.environ.get("LMI_CIMOM_USERNAME", "root")
        cls.password = os.environ.get("LMI_CIMOM_PASSWORD", "")
        cls.disk = os.environ.get("LMI_STORAGE_DISK", "")
        cls.partitions = os.environ.get("LMI_STORAGE_PARTITIONS", "").split()
        cls.cimom = os.environ.get("LMI_CIMOM_BROKER", "sblim-sfcb")
        cls.clean = os.environ.get("LMI_STORAGE_CLEAN", "Yes")
        cls.verbose = os.environ.get("LMI_STORAGE_VERBOSE", None)
        cls.mnt_dir = os.environ.get("LMI_STORAGE_MNT_DIR", "/mnt")
        cls.mydir = os.path.dirname(__file__)
        cls.wbemconnection = pywbem.WBEMConnection(cls.url, (cls.username, cls.password))

        elems = cls.wbemconnection.ExecQuery("WQL",
                    'select * from CIM_StorageExtent where Name="'
                    + cls.disk + '"')
        if elems:
            cls.disk_name = elems[0].path

        cls.partition_names = []
        for device in cls.partitions:
            elems = cls.wbemconnection.ExecQuery("WQL",
                    'select * from CIM_StorageExtent where Name="'
                    + device + '"')
            if elems:
                cls.partition_names.append(elems[0].path)

        # get correct system name
        cls.SYSTEM_NAME = cls.disk_name['SystemName']

    def setUp(self):
        self.start_udev_monitor()

    def start_udev_monitor(self):
        # pylint: disable-msg=W0201
        self.udev_context = pyudev.Context()
        self.udev_monitor = pyudev.Monitor.from_netlink(self.udev_context)
        self.udev_monitor.filter_by('block')
        self.udev_observer = pyudev.MonitorObserver(
                self.udev_monitor, self.udev_event)
        self.devices = []
        self.udev_observer.start()

    def stop_udev_monitor(self):
        self.udev_observer.stop()

    def udev_event(self, action, device):
        if action == 'change':
            return

        if self.verbose:
            print "UDEV event:", action, device.device_node
        if action == 'add':
            self.devices.append(device.device_node)
        if action == 'remove':
            if device.device_node in self.devices:
                self.devices.remove(device.device_node)

    def destroy_created(self):
        """
            Destroy all devices created during the tests as recorded by the
            udev. It returns nr. of removed items.
        """
        count = 0

        # remove them in reverse order
        while self.devices:
            device = self.devices.pop()
            udev_device = pyudev.Device.from_device_file(
                    self.udev_context, device)
            if self.verbose:
                print "Destroying %s:%s" % (device, udev_device.device_type)

            if udev_device.device_type == 'partition':
                count += 1
                self.destroy_mbr(udev_device.parent.device_node)
            if udev_device.device_type == 'disk':
                # is it RAID?
                try:
                    if udev_device['MD_LEVEL']:
                        count += 1
                        self.destroy_md(device)
                except KeyError:
                    pass  # it's not RATD

            # TODO: add LVM
        return count


    def destroy_vg(self, vgname):
        """
            Destroy given volume group, not using CIM.
            This method should be called when a test fails and wants to clean
            up its mess.
        """
        return self.log_run(["vgremove", "-f", "/dev/mapper/" + vgname])

    def destroy_md(self, md_device_id):
        """
            Destroy given RAID, not using CIM.
            This method should be called when a test fails and wants to clean
            up its mess.
        """
        return self.log_run([self.mydir + "/tools/mdremove", md_device_id])

    def destroy_mbr(self, disk_device_id):
        """
            Destroy any partition table on given device.
            This method should be called when a test fails and wants to clean
            up its mess.
        """
        return self.log_run([self.mydir + "/tools/mbrremove", disk_device_id])

    def restart_cim(self):
        """
            Restart CIMOM
        """
        ret = self.log_run(["systemctl", "restart", self.cimom])
        time.sleep(1)
        if ret == 0:
            self.wbemconnection = pywbem.WBEMConnection(
                    self.url, (self.username, self.password))
        return ret



    def log_run(self, args):
        """
            Print arguments and run them.
            args must be prepared for subprocess.call()
        """
        print "Running:", " ".join(args)
        return subprocess.call(args)


    def tearDown(self):
        """
            Default teardown. It destroys any devices created since setUp().
            It restarts CIMOM if any devices were destroyed.
            
            Each test should clean after itself!
        """
        # try to destroy everything and restart CIMOM
        self.stop_udev_monitor()
        if self.clean:
            if self.destroy_created():
                self.restart_cim()

    def _check_redundancy(self, extent, setting,
            data_redundancy=None,
            stripe_legtht=None,
            package_redundancy=None,
            parity_layout=None,
            check_parity_layout=False,
            nspof=None
            ):
        """
            Check if redundancy setting of StorageExtent and StorageSetting
            match and have requested values. Assert if not.
            If any value is None, it will not be checked, except parity_layout.
            If check_parity_layout is True, parity_layout will be checked
            against setting['ParityLayout'] even if it is None 
            Both extent and setting must be CIMInstance.
        """
        self.assertEqual(
                setting['DataRedundancyGoal'], extent['DataRedundancy'])
        self.assertEqual(
                setting['DataRedundancyMin'], extent['DataRedundancy'])
        self.assertEqual(
                setting['DataRedundancyMax'], extent['DataRedundancy'])
        self.assertEqual(
                setting['ExtentStripeLength'], extent['ExtentStripeLength'])
        self.assertEqual(
                setting['ExtentStripeLengthMin'], extent['ExtentStripeLength'])
        self.assertEqual(
                setting['ExtentStripeLengthMax'], extent['ExtentStripeLength'])
        self.assertEqual(
                setting['NoSinglePointOfFailure'],
                extent['NoSinglePointOfFailure'])
        self.assertEqual(
                setting['PackageRedundancyGoal'], extent['PackageRedundancy'])
        self.assertEqual(
                setting['PackageRedundancyMin'], extent['PackageRedundancy'])
        self.assertEqual(
                setting['PackageRedundancyMax'], extent['PackageRedundancy'])

        if data_redundancy:
            self.assertEqual(extent['DataRedundancy'], data_redundancy)
        if stripe_legtht:
            self.assertEqual(extent['ExtentStripeLength'], stripe_legtht)
        if package_redundancy:
            self.assertEqual(extent['PackageRedundancy'], package_redundancy)
        if check_parity_layout or parity_layout:
            self.assertEqual(setting['ParityLayout'], parity_layout)
        if nspof is not None:
            self.assertEqual(setting['NoSinglePointOfFailure'], nspof)

    def finish_job(self, jobname, return_constructor=int,
            affected_output_name=None):
        """
        Wait until the job finishes and return (ret, outparams) as if
        InvokeMethod without a job was returned.
        
        It's hard to reconstruct these outparams, since the embedded instances/
        ojects do not work in our CIMOMS, therefore special care is needed.
        
        When affected_output_name is specified, this methods finds the first
        element associated to the job using LMI_AffestedStorageJobElement and
        puts it into oputparams as affected_output_name entry.
        
        :param jobname: (``CIMInstanceName``) Name of the job.
        :param return_constructor: (function) Function, which converts
            string to the right type, for example int.
        :param affected_output_name: (``string``) If the output parameter of
            the method can discovered using LMI_<name>AffectedMethodElement
            association, this is the name of output parameter, which corresponds
            to this affected element.
        """
        # Use busy loop for now
        # TODO: rework to something sane
        while True:
            job = self.wbemconnection.GetInstance(jobname)
            if job['JobState'] > 5:  # all states higher than 5 are final
                break
            time.sleep(0.1)

        # get the MethodResult
        resultname = self.wbemconnection.AssociatorNames(jobname,
                AssocClass="LMI_AssociatedStorageJobMethodResult")[0]
        result = self.wbemconnection.GetInstance(resultname)
        ind = result['PostCallIndication']
        # check for error
        if ind['Error'] is not None:
            err = ind['Error'][0]
            code = err['CIMStatusCode']
            msg = err['Message']
            raise pywbem.CIMError(code, msg)

        ret = return_constructor(ind['ReturnValue'])

        # convert output parameters to format returned by InvokeMethod
        outparams = pywbem.NocaseDict()
        try:
            params = ind['MethodParameters']
        except KeyError:
            params = {}
        if params:
            for (key, value) in params.iteritems():
                outparams[key] = value

        return (ret, outparams)

    def _compare_cim_name(self, first, second):
        """
        Compare two CIMInstanceNames. Their hostname is not checked.
        """
        first_host = first.host
        first.host = None
        second_host = second.host
        second.host = None

        equals = first == second
        first.host = first_host
        second.host = second_host
        return equals

    def assertCIMNameEquals(self, first, second):
        """
        Compare two CIMInstanceNames. Their hostname is not checked.
        """
        self.assertTrue(self._compare_cim_name(first, second))

    def assertCIMNameIn(self, name, candidates):
        """
        Checks that given CIMInstanceName is in given set. It compares all
        properties except hostname.
        """
        for candidate in candidates:
            if self._compare_cim_name(name, candidate):
                return
        self.assertTrue(False, "name is not in candidates")

    def assertNocaseDictEqual(self, a, b, msg=None):
        a1 = {}
        for (key, value) in a.iteritems():
            a1[key.lower()] = value

        b1 = {}
        for (key, value) in b.iteritems():
            b1[key.lower()] = value

        self.assertDictEqual(a1, b1, msg)

    def invoke_async_method(self,
            method_name,
            object_name,
            return_constructor=int,
            affected_output_name=None,
            *args, **kwargs):
        """
        Invoke a method and if it returns a job, wait for the job.
        Return (retvalue, outparams) in the same way as finish_method() would.

        :param method_name: (``string``) Name of the method.
        :param object_name: (``CIMInstanceName``) Instance, on which the method
            should be invoked.
        :param return_constructor: (function) Function, which converts
            string to the right type, for example int.
        :param affected_output_name: (``string``) If the output parameter of
            the method can discovered using LMI_<name>AffectedMethodElement
            association, this is the name of output parameter, which corresponds
            to this affected element.
        """
        (ret, outparams) = self.wbemconnection.InvokeMethod(
                method_name,
                object_name,
                *args,
                **kwargs)
        if ret == self.JOB_CREATED:
            # wait for the job
            jobname = outparams['Job']
            (ret, outparams) = self.finish_job(jobname,
                    return_constructor,
                    affected_output_name)
        return (ret, outparams)


    def _create_mdraid(self, devicenames, level):
        """
        Create MD RAID device from given devices.

        :param devicenames: (``array of CIMInstanceNames``) RAID member devices.
        :param level: (int) RAID level

        :returns: ``CIMInstanceName`` of the RAID.
        """
        storage_service = self.wbemconnection.EnumerateInstanceNames(
                "LMI_StorageConfigurationService")[0]
        (ret, outparams) = self.invoke_async_method(
                "CreateOrModifyMDRAID",
                storage_service,
                int, "theelement",
                InExtents=devicenames,
                Level=pywbem.Uint16(level))
        self.assertEqual(ret, 0)
        return outparams['theelement']


    def _delete_mdraid(self, raidname):
        """
        Delete MD RAID.

        :param raidname: (``CIMInstanceName``) RAID to delete.
        """
        storage_service = self.wbemconnection.EnumerateInstanceNames(
                "LMI_StorageConfigurationService")[0]
        (ret, _outparams) = self.invoke_async_method(
                "DeleteMDRAID",
                storage_service,
                int,
                TheElement=raidname)
        self.assertEquals(ret, 0)

def short_tests_only():
    """
        Returns True, if only short test should be executed, i.e.
        LMI_STORAGE_SHORT_ONLY is set.
    """
    if os.environ.get("LMI_STORAGE_SHORT_ONLY", None):
        return True
    return False



#!/usr/bin/python
# -*- Coding:utf-8 -*-
#
# Copyright (C) 2012 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Jan Safranek <jsafrane@redhat.com>

from test_base import StorageTestBase
import unittest
import pywbem

MEGABYTE = 1024 * 1024

class TestCreateMDRAID(StorageTestBase):
    """
        Test LMI_StorageConfigurationService.TestCreateMDRAID
        (create only).
    """

    STYLE_GPT = 3
    PARTITION_CLASS = "LMI_GenericDiskPartition"


    def setUp(self):
        """ Find storage service. """
        super(TestCreateMDRAID, self).setUp()
        self.service = self.wbemconnection.EnumerateInstanceNames(
                "LMI_StorageConfigurationService")[0]
        self.part_service = self.wbemconnection.EnumerateInstanceNames(
                "LMI_DiskPartitionConfigurationService")[0]
        self.capabilities = self.wbemconnection.EnumerateInstanceNames(
                "LMI_MDRAIDStorageCapabilities")[0]

    def _create_setting(self, level=None, devices=None):
        """ Create a MDRAIDStorageSetting and return CIMInstance of it."""
        if level is None:
            (retval, outparams) = self.wbemconnection.InvokeMethod(
                    "CreateSetting",
                    self.capabilities)
            self.assertEqual(retval, 0)
            self.assertEqual(len(outparams), 1)

            setting_name = outparams['newsetting']
            setting = self.wbemconnection.GetInstance(setting_name)
            return setting

        # create settign for given RAID level with given devices
        (retval, outparams) = self.wbemconnection.InvokeMethod(
                "CreateMDRAIDStorageSetting",
                self.capabilities,
                Level=pywbem.Uint16(level),
                InExtents=devices)
        self.assertEqual(retval, 0)
        self.assertEqual(len(outparams), 1)

        setting_name = outparams['setting']
        setting = self.wbemconnection.GetInstance(setting_name)
        return setting

    def _create_empty_setting(self):
        """ Create a MDRAIDStorageSetting, remove all redundancy properties
            from it and return it
        ."""
        setting = self._create_setting()

        setting['DataRedundancyGoal'] = pywbem.CIMProperty(
                name='DataRedundancyGoal',
                value=None,
                type='uint16')
        setting['DataRedundancyMax'] = pywbem.CIMProperty(
                name='DataRedundancyMax',
                value=None,
                type='uint16')
        setting['DataRedundancyMin'] = pywbem.CIMProperty(
                name='DataRedundancyMin',
                value=None,
                type='uint16')
        setting['PackageRedundancyGoal'] = pywbem.CIMProperty(
                name='PackageRedundancyGoal',
                value=None,
                type='uint16')
        setting['PackageRedundancyMin'] = pywbem.CIMProperty(
                name='PackageRedundancyMin',
                value=None,
                type='uint16')
        setting['PackageRedundancyMax'] = pywbem.CIMProperty(
                name='PackageRedundancyMax',
                value=None,
                type='uint16')
        setting['ExtentStripeLength'] = pywbem.CIMProperty(
                name='ExtentStripeLength',
                value=None,
                type='uint16')
        setting['ExtentStripeLengthMin'] = pywbem.CIMProperty(
                name='ExtentStripeLengthMin',
                value=None,
                type='uint16')
        setting['ExtentStripeLengthMax'] = pywbem.CIMProperty(
                name='ExtentStripeLengthMax',
                value=None,
                type='uint16')
        setting['NoSinglePointOfFailure'] = pywbem.CIMProperty(
                name='NoSinglePointOfFailure',
                value=None,
                type='boolean')
        setting['ParityLayout'] = pywbem.CIMProperty(
                name='ParityLayout',
                value=None,
                type='uint16')
        self.wbemconnection.ModifyInstance(setting)
        return setting

    def _delete_setting(self, setting_name):
        """ Delete given setting. """
        self.wbemconnection.DeleteInstance(setting_name)

    def _get_extent_size(self, extent_name):
        """ Return size of given disk, in bytes."""
        disk = self.wbemconnection.GetInstance(extent_name)
        return disk['NumberOfBlocks'] * disk['BlockSize']

    def _test_raid_n(self, devices, level,
            data_redundancy, stripe_length, package_redundancy, parity_layout,
            nspof, expected_size, goal=None):
        """
            Create RAID<level> with given devices.
            Then check that it has specified redundancy properties.
            Destroy everything.
        """
        if goal:
            (ret, outparams) = self.invoke_async_method(
                    "CreateOrModifyMDRAID",
                    self.service,
                    int, "theelement",
                    InExtents=devices,
                    Goal=goal)
        else:
            (ret, outparams) = self.invoke_async_method(
                    "CreateOrModifyMDRAID",
                    self.service,
                    int, "theelement",
                    InExtents=devices,
                    Level=pywbem.Uint16(level))
        if len(outparams) == 1:
            # there is no Size returned, Pegasus does not support it yet
            # TODO: remove when pegasus supports embedded objects of unknown
            # types, rhbz#920763
            if outparams.has_key('theelement'):
                raid = self.wbemconnection.GetInstance(outparams['theelement'])
                outparams['size'] = raid['NumberOfBlocks'] * raid['BlockSize']

        self.assertEqual(ret, 0)
        self.assertEqual(len(outparams), 2)
        self.assertAlmostEqual(
                outparams['size'],
                expected_size,
                delta=4 * MEGABYTE)
        raidname = outparams['theelement']
        raid = self.wbemconnection.GetInstance(raidname)
        self.assertNotEqual(raid['UUID'], '')
        self.assertEqual(raid['Level'], level)
        self.assertNotEqual(raid['UUID'], None)
        self.assertEqual(raid['Primordial'], False)

        # check based-on
        basedons = self.wbemconnection.References(
                raidname,
                ResultClass="LMI_MDRAIDBasedOn")
        self.assertEqual(len(basedons), len(devices))
        for basedon in basedons:
            self.assertCIMNameIn(basedon['Antecedent'], devices)
            self.assertCIMNameEquals(basedon['Dependent'], raidname)

        # check setting
        settings = self.wbemconnection.Associators(
                raidname,
                AssocClass="LMI_MDRAIDElementSettingData")
        self.assertEqual(len(settings), 1)
        setting = settings[0]

        self._check_redundancy(raid, setting,
                data_redundancy=data_redundancy,
                stripe_legtht=stripe_length,
                package_redundancy=package_redundancy,
                parity_layout=parity_layout,
                check_parity_layout=True,
                nspof=nspof)

        # delete the raid
        (ret, outparams) = self.invoke_async_method(
                "DeleteMDRAID",
                self.service,
                int,
                TheElement=raidname)
        self.assertEquals(ret, 0)

    def test_create_raid0_level(self):
        """ Test CreateOrModifyMDRAID level 0."""
        partition_size = self._get_extent_size(self.partition_names[0])
        # two devices
        self._test_raid_n(self.partition_names[:2], level=0,
                data_redundancy=1,
                stripe_length=2,
                package_redundancy=0,
                parity_layout=None,
                nspof=False,
                expected_size=partition_size * 2)
        # three devices
        self._test_raid_n(self.partition_names[:3], level=0,
                data_redundancy=1,
                stripe_length=3,
                package_redundancy=0,
                parity_layout=None,
                nspof=False,
                expected_size=partition_size * 3)

    def test_create_raid0_goal(self):
        """ Test CreateOrModifyMDRAID level 0 with Goal."""
        partition_size = self._get_extent_size(self.partition_names[0])
        # two devices
        setting = self._create_setting(0, self.partition_names[:2])
        self._test_raid_n(self.partition_names[:2], level=0,
                data_redundancy=1,
                stripe_length=2,
                package_redundancy=0,
                parity_layout=None,
                nspof=False,
                expected_size=partition_size * 2,
                goal=setting.path)
        self._delete_setting(setting.path)
        # three devices
        setting = self._create_setting(0, self.partition_names[:3])
        self._test_raid_n(self.partition_names[:3], level=0,
                data_redundancy=1,
                stripe_length=3,
                package_redundancy=0,
                parity_layout=None,
                nspof=False,
                expected_size=partition_size * 3,
                goal=setting.path)
        self._delete_setting(setting.path)

    def test_create_raid1_level(self):
        """ Test CreateOrModifyMDRAID level 1."""
        partition_size = self._get_extent_size(self.partition_names[0])

        # two devices
        self._test_raid_n(self.partition_names[:2], level=1,
                data_redundancy=2,
                stripe_length=1,
                package_redundancy=1,
                parity_layout=None,
                nspof=True,
                expected_size=partition_size)
        # three devices
        self._test_raid_n(self.partition_names[:4], level=1,
                data_redundancy=4,
                stripe_length=1,
                package_redundancy=3,
                parity_layout=None,
                nspof=True,
                expected_size=partition_size)

    def test_create_raid1_goal(self):
        """ Test CreateOrModifyMDRAID level 1 with Goal."""
        partition_size = self._get_extent_size(self.partition_names[0])

        # two devices
        setting = self._create_setting(1, self.partition_names[:2])
        self._test_raid_n(self.partition_names[:2], level=1,
                data_redundancy=2,
                stripe_length=1,
                package_redundancy=1,
                parity_layout=None,
                nspof=True,
                expected_size=partition_size,
                goal=setting.path)
        self._delete_setting(setting.path)
        # three devices
        setting = self._create_setting(1, self.partition_names[:4])
        self._test_raid_n(self.partition_names[:4], level=1,
                data_redundancy=4,
                stripe_length=1,
                package_redundancy=3,
                parity_layout=None,
                nspof=True,
                expected_size=partition_size,
                goal=setting.path)
        self._delete_setting(setting.path)

    def test_create_raid4_level(self):
        """ Test CreateOrModifyMDRAID level 4."""
        partition_size = self._get_extent_size(self.partition_names[0])

        # 3 devices
        self._test_raid_n(self.partition_names[:3], level=4,
                data_redundancy=1,
                stripe_length=3,
                package_redundancy=1,
                parity_layout=1,  # non-rotated parity
                nspof=True,
                expected_size=partition_size * 2)
        # 4 devices
        self._test_raid_n(self.partition_names[:4], level=4,
                data_redundancy=1,
                stripe_length=4,
                package_redundancy=1,
                parity_layout=1,  # non-rotated parity
                nspof=True,
                expected_size=partition_size * 3)

    def test_create_raid4_goal(self):
        """ Test CreateOrModifyMDRAID level 4 with Goal."""
        partition_size = self._get_extent_size(self.partition_names[0])

        # 3 devices
        setting = self._create_setting(4, self.partition_names[:3])
        self._test_raid_n(self.partition_names[:3], level=4,
                data_redundancy=1,
                stripe_length=3,
                package_redundancy=1,
                parity_layout=1,  # non-rotated parity
                nspof=True,
                expected_size=partition_size * 2,
                goal=setting.path)
        self._delete_setting(setting.path)
        # 4 devices
        setting = self._create_setting(4, self.partition_names[:4])
        self._test_raid_n(self.partition_names[:4], level=4,
                data_redundancy=1,
                stripe_length=4,
                package_redundancy=1,
                parity_layout=1,  # non-rotated parity
                nspof=True,
                expected_size=partition_size * 3,
                goal=setting.path)
        self._delete_setting(setting.path)

    def test_create_raid5_level(self):
        """ Test CreateOrModifyMDRAID level 5."""
        partition_size = self._get_extent_size(self.partition_names[0])

        # 3 devices
        self._test_raid_n(self.partition_names[:3], level=5,
                data_redundancy=1,
                stripe_length=3,
                package_redundancy=1,
                parity_layout=2,  # rotated parity
                nspof=True,
                expected_size=partition_size * 2)
        # 5 devices
        self._test_raid_n(self.partition_names[:5], level=5,
                data_redundancy=1,
                stripe_length=5,
                package_redundancy=1,
                parity_layout=2,  # rotated parity
                nspof=True,
                expected_size=partition_size * 4)

    def test_create_raid5_goal(self):
        """ Test CreateOrModifyMDRAID level 5 with Goal."""
        partition_size = self._get_extent_size(self.partition_names[0])

        # 3 devices
        setting = self._create_setting(5, self.partition_names[:3])
        self._test_raid_n(self.partition_names[:3], level=5,
                data_redundancy=1,
                stripe_length=3,
                package_redundancy=1,
                parity_layout=2,  # rotated parity
                nspof=True,
                expected_size=partition_size * 2,
                goal=setting.path)
        self._delete_setting(setting.path)
        # 5 devices
        setting = self._create_setting(5, self.partition_names[:5])
        self._test_raid_n(self.partition_names[:5], level=5,
                data_redundancy=1,
                stripe_length=5,
                package_redundancy=1,
                parity_layout=2,  # rotated parity
                nspof=True,
                expected_size=partition_size * 4,
                goal=setting.path)
        self._delete_setting(setting.path)

    def test_create_raid6_level(self):
        """ Test CreateOrModifyMDRAID level 6."""
        partition_size = self._get_extent_size(self.partition_names[0])

        # 4 devices
        self._test_raid_n(self.partition_names[:4], level=6,
                data_redundancy=1,
                stripe_length=4,
                package_redundancy=2,
                parity_layout=2,  # rotated parity
                nspof=True,
                expected_size=partition_size * 2)
        # 5 devices
        self._test_raid_n(self.partition_names[:5], level=6,
                data_redundancy=1,
                stripe_length=5,
                package_redundancy=2,
                parity_layout=2,  # rotated parity
                nspof=True,
                expected_size=partition_size * 3)

    def test_create_raid6_goal(self):
        """ Test CreateOrModifyMDRAID level 6 with Goal."""
        partition_size = self._get_extent_size(self.partition_names[0])

        # 4 devices
        setting = self._create_setting(6, self.partition_names[:4])
        self._test_raid_n(self.partition_names[:4], level=6,
                data_redundancy=1,
                stripe_length=4,
                package_redundancy=2,
                parity_layout=2,  # rotated parity
                nspof=True,
                expected_size=partition_size * 2,
                goal=setting.path)
        self._delete_setting(setting.path)

        # 5 devices
        setting = self._create_setting(6, self.partition_names[:5])
        self._test_raid_n(self.partition_names[:5], level=6,
                data_redundancy=1,
                stripe_length=5,
                package_redundancy=2,
                parity_layout=2,  # rotated parity
                nspof=True,
                expected_size=partition_size * 3,
                goal=setting.path)
        self._delete_setting(setting.path)

    def test_create_raid10_level(self):
        """ Test CreateOrModifyMDRAID level 10."""
        partition_size = self._get_extent_size(self.partition_names[0])

        # 4 devices
        self._test_raid_n(self.partition_names[:4], level=10,
                data_redundancy=2,
                stripe_length=2,
                package_redundancy=1,
                parity_layout=None,
                nspof=True,
                expected_size=partition_size * 2)
        # 5 devices
        self._test_raid_n(self.partition_names[:5], level=10,
                data_redundancy=2,
                stripe_length=3,
                package_redundancy=1,
                parity_layout=None,
                nspof=True,
                expected_size=partition_size * 2.5)
        # 6 devices
        self._test_raid_n(self.partition_names[:6], level=10,
                data_redundancy=2,
                stripe_length=3,
                package_redundancy=1,
                parity_layout=None,
                nspof=True,
                expected_size=partition_size * 3)

    def test_create_raid10_goal(self):
        """ Test CreateOrModifyMDRAID level 10 with a valid Goal."""
        partition_size = self._get_extent_size(self.partition_names[0])

        # 4 devices
        setting = self._create_setting(10, self.partition_names[:4])
        self._test_raid_n(self.partition_names[:4], level=10,
                data_redundancy=2,
                stripe_length=2,
                package_redundancy=1,
                parity_layout=None,
                nspof=True,
                expected_size=partition_size * 2,
                goal=setting.path)
        self._delete_setting(setting.path)

        # 5 devices
        setting = self._create_setting(10, self.partition_names[:5])
        self._test_raid_n(self.partition_names[:5], level=10,
                data_redundancy=2,
                stripe_length=3,
                package_redundancy=1,
                parity_layout=None,
                nspof=True,
                expected_size=partition_size * 2.5,
                goal=setting.path)
        self._delete_setting(setting.path)

        # 6 devices
        setting = self._create_setting(10, self.partition_names[:6])
        self._test_raid_n(self.partition_names[:6], level=10,
                data_redundancy=2,
                stripe_length=3,
                package_redundancy=1,
                parity_layout=None,
                nspof=True,
                expected_size=partition_size * 3,
                goal=setting.path)
        self._delete_setting(setting.path)

    def test_create_wrong_device_numbers(self):
        """
            Test CreateOrModifyMDRAID with various levels and bad nr. of
            devices.
        """
        # raid 0 with 1 device
        self.assertRaises(pywbem.CIMError, self.wbemconnection.InvokeMethod,
                "CreateOrModifyMDRAID",
                self.service,
                InExtents=self.partition_names[:1],
                Level=pywbem.Uint16(0))

        # raid 1 with 1 device
        self.assertRaises(pywbem.CIMError, self.wbemconnection.InvokeMethod,
                "CreateOrModifyMDRAID",
                self.service,
                InExtents=self.partition_names[:1],
                Level=pywbem.Uint16(1))

        # raid 4 with 1 device
        self.assertRaises(pywbem.CIMError, self.wbemconnection.InvokeMethod,
                "CreateOrModifyMDRAID",
                self.service,
                InExtents=self.partition_names[:1],
                Level=pywbem.Uint16(4))

        # raid 4 with 2 devices
        self.assertRaises(pywbem.CIMError, self.wbemconnection.InvokeMethod,
                "CreateOrModifyMDRAID",
                self.service,
                InExtents=self.partition_names[:2],
                Level=pywbem.Uint16(4))

        # raid 5 with 1 device
        self.assertRaises(pywbem.CIMError, self.wbemconnection.InvokeMethod,
                "CreateOrModifyMDRAID",
                self.service,
                InExtents=self.partition_names[:1],
                Level=pywbem.Uint16(5))

        # raid 5 with 2 devices
        self.assertRaises(pywbem.CIMError, self.wbemconnection.InvokeMethod,
                "CreateOrModifyMDRAID",
                self.service,
                InExtents=self.partition_names[:2],
                Level=pywbem.Uint16(5))

        # raid 6 with 1 device
        self.assertRaises(pywbem.CIMError, self.wbemconnection.InvokeMethod,
                "CreateOrModifyMDRAID",
                self.service,
                InExtents=self.partition_names[:1],
                Level=pywbem.Uint16(6))

        # raid 6 with 2 devices
        self.assertRaises(pywbem.CIMError, self.wbemconnection.InvokeMethod,
                "CreateOrModifyMDRAID",
                self.service,
                InExtents=self.partition_names[:2],
                Level=pywbem.Uint16(6))

        # raid 6 with 3 devices
        self.assertRaises(pywbem.CIMError, self.wbemconnection.InvokeMethod,
                "CreateOrModifyMDRAID",
                self.service,
                InExtents=self.partition_names[:3],
                Level=pywbem.Uint16(6))

        # raid 10 with 1 device
        self.assertRaises(pywbem.CIMError, self.wbemconnection.InvokeMethod,
                "CreateOrModifyMDRAID",
                self.service,
                InExtents=self.partition_names[:1],
                Level=pywbem.Uint16(10))



    def test_create_wrong_goals(self):
        """
            Test CreateOrModifyMDRAID with various wrong goals.
        """
        # unknown goal
        goal = pywbem.CIMInstanceName(
                classname=" LMI_MDRAIDStorageSetting",
                keybindings={
                        'InstanceID' : 'LMI:LMI_MDRAIDStorageSetting:not-existing'
        })
        self.assertRaises(pywbem.CIMError, self.wbemconnection.InvokeMethod,
                "CreateOrModifyMDRAID",
                self.service,
                InExtents=self.partition_names[:2],
                Level=pywbem.Uint16(0),
                Goal=goal)

        # raid1 with wrong data redundancy
        setting = self._create_setting(1, self.partition_names[:2])
        setting['DataRedundancyGoal'] = pywbem.Uint16(3)
        setting['DataRedundancyMin'] = pywbem.Uint16(3)
        setting['DataRedundancyMax'] = pywbem.Uint16(3)
        self.wbemconnection.ModifyInstance(setting)
        self.assertRaises(pywbem.CIMError, self.wbemconnection.InvokeMethod,
                "CreateOrModifyMDRAID",
                self.service,
                InExtents=self.partition_names[:2],
                Goal=setting.path)
        self._delete_setting(setting.path)

        # raid1 with wrong package redundancy
        setting = self._create_setting(1, self.partition_names[:2])
        setting['PackageRedundancyGoal'] = pywbem.Uint16(2)
        setting['PackageRedundancyMin'] = pywbem.Uint16(2)
        setting['PackageRedundancyMax'] = pywbem.Uint16(2)
        self.wbemconnection.ModifyInstance(setting)
        self.assertRaises(pywbem.CIMError, self.wbemconnection.InvokeMethod,
                "CreateOrModifyMDRAID",
                self.service,
                InExtents=self.partition_names[:2],
                Goal=setting.path)
        self._delete_setting(setting.path)

    def test_create_calculate_goals(self):
        """
            Test CreateOrModifyMDRAID with Goal settings, where not all
            properties are set and the implementation must choose the
            best RAID level.
        """
        partition_size = self._get_extent_size(self.partition_names[0])

        # raid1 is selected as default if no properties are set
        setting = self._create_empty_setting()
        self._test_raid_n(self.partition_names[:4], level=1,
                data_redundancy=4,
                stripe_length=1,
                package_redundancy=3,
                parity_layout=None,
                nspof=True,
                expected_size=partition_size * 1,
                goal=setting.path)
        self._delete_setting(setting.path)

        #  ParityLayout set -> use RAID5 (and not RAID6)
        setting = self._create_empty_setting()
        setting['ParityLayout'] = pywbem.Uint16(2)  # Rotated parity
        self.wbemconnection.ModifyInstance(setting)
        self._test_raid_n(self.partition_names[:4], level=5,
                data_redundancy=1,
                stripe_length=4,
                package_redundancy=1,
                parity_layout=2,
                nspof=True,
                expected_size=partition_size * 3,
                goal=setting.path)
        self._delete_setting(setting.path)

        #  ParityLayout set and higher redundancy -> use RAID6
        setting = self._create_empty_setting()
        setting['ParityLayout'] = pywbem.Uint16(2)  # Rotated parity
        setting['PackageRedundancyMin'] = pywbem.Uint16(2)
        self.wbemconnection.ModifyInstance(setting)
        self._test_raid_n(self.partition_names[:4], level=6,
                data_redundancy=1,
                stripe_length=4,
                package_redundancy=2,
                parity_layout=2,
                nspof=True,
                expected_size=partition_size * 2,
                goal=setting.path)
        self._delete_setting(setting.path)

        #  ParityLayout set  -> use RAID4
        setting = self._create_empty_setting()
        setting['ParityLayout'] = pywbem.Uint16(1)  # Non-Rotated parity
        self.wbemconnection.ModifyInstance(setting)
        self._test_raid_n(self.partition_names[:4], level=4,
                data_redundancy=1,
                stripe_length=4,
                package_redundancy=1,
                parity_layout=1,
                nspof=True,
                expected_size=partition_size * 3,
                goal=setting.path)
        self._delete_setting(setting.path)

        #  request higher redundancy -> RAID1 (instead of RAID 4, 5, 6, 10)
        setting = self._create_empty_setting()
        setting['PackageRedundancyMin'] = pywbem.Uint16(1)
        self.wbemconnection.ModifyInstance(setting)
        self._test_raid_n(self.partition_names[:4], level=1,
                data_redundancy=4,
                stripe_length=1,
                package_redundancy=3,
                parity_layout=None,
                nspof=True,
                expected_size=partition_size * 1,
                goal=setting.path)
        self._delete_setting(setting.path)

        #  request higher data and pkg. redundancy -> RAID1 (instead of RAID10)
        setting = self._create_empty_setting()
        setting['PackageRedundancyMin'] = pywbem.Uint16(1)
        setting['DataRedundancyMin'] = pywbem.Uint16(2)
        self.wbemconnection.ModifyInstance(setting)
        self._test_raid_n(self.partition_names[:4], level=1,
                data_redundancy=4,
                stripe_length=1,
                package_redundancy=3,
                parity_layout=None,
                nspof=True,
                expected_size=partition_size * 1,
                goal=setting.path)
        self._delete_setting(setting.path)

        #  request higher data and pkg. redundancy with upper limit -> RAID10 only
        setting = self._create_empty_setting()
        setting['PackageRedundancyMin'] = pywbem.Uint16(1)
        setting['DataRedundancyMin'] = pywbem.Uint16(2)
        setting['DataRedundancyMax'] = pywbem.Uint16(3)
        self.wbemconnection.ModifyInstance(setting)
        self._test_raid_n(self.partition_names[:4], level=10,
                data_redundancy=2,
                stripe_length=2,
                package_redundancy=1,
                parity_layout=None,
                nspof=True,
                expected_size=partition_size * 2,
                goal=setting.path)
        self._delete_setting(setting.path)

        #  request higher redundancy with upper limit -> RAID5 (instead of 6 and 4)
        setting = self._create_empty_setting()
        setting['PackageRedundancyMin'] = pywbem.Uint16(1)
        setting['PackageRedundancyMax'] = pywbem.Uint16(2)
        self.wbemconnection.ModifyInstance(setting)
        self._test_raid_n(self.partition_names[:4], level=5,
                data_redundancy=1,
                stripe_length=4,
                package_redundancy=1,
                parity_layout=2,
                nspof=True,
                expected_size=partition_size * 3,
                goal=setting.path)
        self._delete_setting(setting.path)

    def test_create_on_disk(self):
        """
        Test MD RAID on plain disk, without partition table.
        """
        devices = [self.partition_names[0], self.disk_name]
        (ret, outparams) = self.invoke_async_method(
                "CreateOrModifyMDRAID",
                self.service,
                int, "theelement",
                InExtents=devices,
                Level=pywbem.Uint16(1))
        self.assertEqual(ret, 0)
        raidname = outparams['theelement']

        # check based-on
        basedons = self.wbemconnection.References(
                raidname,
                ResultClass="LMI_MDRAIDBasedOn")
        self.assertEqual(len(basedons), 2)
        for basedon in basedons:
            self.assertCIMNameIn(basedon['Antecedent'], devices)
            self.assertCIMNameEquals(basedon['Dependent'], raidname)

        # destroy it
        (ret, outparams) = self.invoke_async_method(
                "DeleteMDRAID",
                self.service,
                int,
                TheElement=raidname)
        self.assertEquals(ret, 0)


if __name__ == '__main__':
    unittest.main()



from setuptools import setup
from distutils.command.sdist import sdist
import os
import sys

class lmi_sdist(sdist):
    """
    Custom 'setup.py sdist' command, which regenerates documentation
    sources when creating a tarball.
    """

    def run(self):
        """
        Regenerate documentation source files and create tarball.
        """
        ret = os.system("make -C doc/admin src")
        if ret:
            print "Failed to compile documentation sources, exiting."
            sys.exit(ret)
        # sdist is old-style class
        sdist.run(self)

setup(
    name='openlmi-storage',
    description='Anaconda Storage Provider',
    author='Jan Safranek',
    author_email='jsafrane@redhat.com',
    url='https://fedorahosted.org/openlmi/',
    version='0.7.1',
    package_dir={'': 'src'},
    namespace_packages = ['lmi'],
    packages=['lmi', 'lmi.storage', 'lmi.storage.util'],
    classifiers=[
        'License :: OSI Approved :: GNU Lesser General Public License v2 or later (LGPLv2+)',
        'Operating System :: POSIX :: Linux',
        'Topic :: System :: Systems Administration',
    ],

    cmdclass={
        'sdist': lmi_sdist
    }
)



# Copyright (C) 2013 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Jan Synacek <jsynacek@redhat.com>
# -*- coding: utf-8 -*-
"""
Python Provider for LMI_AttachedFileSystem

LMI_AttachedFileSystem
----------------------

.. autoclass:: LMI_AttachedFileSystem
    :members:

"""

import pywbem
import blivet
from lmi.storage.MountingProvider import MountingProvider
import lmi.providers.cmpi_logging as cmpi_logging
from lmi.providers.ComputerSystem import get_system_name

class LMI_AttachedFileSystem(MountingProvider):
    """Instrument the CIM class LMI_AttachedFileSystem

    CIM_Dependency is a generic association used to establish dependency
    relationships between ManagedElements.
    """

    @cmpi_logging.trace_method
    def __init__ (self, *args, **kwargs):
        super(LMI_AttachedFileSystem, self).__init__(*args, **kwargs)
        self.classname = 'LMI_AttachedFileSystem'

    @cmpi_logging.trace_method
    def get_instance(self, env, model):
        """
            Provider implementation of GetInstance intrinsic method.
        """
        fs = model['Antecedent']
        if (fs['CSCreationClassName'] != self.config.system_class_name or
            fs['CSName'] != get_system_name()):
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND,
                                  "Wrong Antecedent keys.")

        spec = model['Dependent']['FileSystemSpec']
        path = model['Dependent']['MountPointPath']

        (device, fmt) = self.get_device_and_format_from_fs(fs)
        self.check_get_instance(device, spec, path)

        return model

    @cmpi_logging.trace_method
    def enum_instances(self, env, model, keys_only):
        """Enumerate instances.
        """
        model.path.update({'Dependent': None, 'Antecedent': None})
        for device in self.storage.devices:
            if isinstance(device, blivet.devices.NoDevice):
                paths = [device.format.mountpoint]
            else:
                paths = blivet.util.get_mount_paths(device.path)

            for path in paths:
                provider = self.provider_manager.get_provider_for_format(device, device.format)
                if provider is None:
                    raise pywbem.CIMError(pywbem.CIM_ERR_FAILED, "Could not get provider for %s" % path)

                model['Antecedent'] = provider.get_name_for_format(device, device.format)
                model['Dependent'] = pywbem.CIMInstanceName(
                    classname='LMI_MountedFileSystem',
                    namespace=self.config.namespace,
                    keybindings={
                        'FileSystemSpec' : device.path,
                        'MountPointPath' : path
                        })

                yield model

    def references(self, env, object_name, model, result_class_name, role,
                   result_role, keys_only):
        return self.simple_references(env, object_name, model,
                                      result_class_name, role, result_role, keys_only,
                                      "LMI_MountedFileSystem",
                                      "CIM_FileSystem")
